package com.taauav.admin.controller;


import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.taauav.admin.entity.SysLevel;
import com.taauav.admin.query.EquipmentQuery;
import com.taauav.common.bean.Response;
import com.taauav.common.constant.PermissionConstants;
import com.taauav.admin.entity.TauvEquipment;
import com.taauav.admin.service.ITauvEquipmentService;
import com.taauav.common.util.DateUtil;
import com.taauav.common.util.FunctionUtils;
import org.apache.commons.lang3.time.DateUtils;
import org.apache.shiro.authz.annotation.RequiresPermissions;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.util.StringUtils;
import org.springframework.validation.BindingResult;
import org.springframework.web.bind.annotation.*;

import javax.validation.Valid;
import java.util.List;
import java.util.Map;

/**
 * <p>
 * 设备管理表 前端控制器
 * </p>
 *
 * @author dyg
 * @since 2019-11-13
 */
@RestController
@RequestMapping("/equipment")
public class TauvEquipmentController {
    @Autowired
    private ITauvEquipmentService iTauvEquipmentService;
    @Autowired
    private Response response;

    private static final String controllerName = "equipment";

    /**
     * 获取飞行设备列表
     *
     * @param query 查询条件
     * @return
     */
    @PostMapping("/index")
    @RequiresPermissions(controllerName + ":" + PermissionConstants.LIST_PERMISSION)
    public Response index(@RequestBody(required = false) EquipmentQuery query) {
        return iTauvEquipmentService.getPageList(query);
    }

    /**
     * 设备编辑
     *
     * @param tauvEquipment
     * @return
     */
    @PostMapping(value = "/edit")
    @RequiresPermissions(controllerName + ":" + PermissionConstants.EDIT_PERMISSION)
    public Response edit(@RequestBody @Valid TauvEquipment tauvEquipment, BindingResult bindingResult) {
        if (bindingResult.hasErrors()) {
            String msg = bindingResult.getFieldError().getDefaultMessage();
            return response.failure(msg);
        }
        return iTauvEquipmentService.edit(tauvEquipment);
    }

    /**
     * 获取设备信息
     *
     * @param id
     * @return
     */
    @GetMapping("/getEquipmentInfo")
    @RequiresPermissions(controllerName + ":" + PermissionConstants.EDIT_PERMISSION)
    public Response getEquipmentInfo(Integer id) {
        TauvEquipment equipment = iTauvEquipmentService.getInfoById(id);
        if (StringUtils.isEmpty(equipment)) {
            return response.failure("数据不存在");
        }
        Map<String, Object> info = FunctionUtils.transBean2Map(equipment);
        info.put("format_register_time", DateUtil.format(equipment.getRegisterTime(), "yyyy-MM-dd HH:mm:ss"));
        info.put("typeList", equipment.getTypeList());
        info.put("statusList", equipment.getStatusList());
        return response.success(info);
    }

    /**
     * 设备添加
     *
     * @param tauvEquipment
     * @return
     */
    @PostMapping("/add")
    @RequiresPermissions(controllerName + ":" + PermissionConstants.ADD_PERMISSION)
    public Response add(@RequestBody @Valid TauvEquipment tauvEquipment, BindingResult bindingResult) {
        if (bindingResult.hasErrors()) {
            String msg = bindingResult.getFieldError().getDefaultMessage();
            return response.failure(msg);
        }
        return iTauvEquipmentService.add(tauvEquipment);
    }

    /**
     * 删除设备
     *
     * @param id 设备ID
     * @return
     */
    @PostMapping("/drop")
    @RequiresPermissions(controllerName + ":" + PermissionConstants.REMOVE_PERMISSION)
    public Response drop(Integer id) {
        return iTauvEquipmentService.drop(id);
    }

    /**
     * 设置状态
     *
     * @param map
     * @return
     */
    @PostMapping("/setStatus")
    @RequiresPermissions(controllerName + ":" + PermissionConstants.STATUS_PERMISSION)
    public Response setStatus(@RequestBody Map<String, String> map) {
        return iTauvEquipmentService.setRules(map);
    }

    /**
     * 基础数据
     *
     * @return
     */
    @GetMapping("/baseData")
    public Response getBaseData() {
        return response.success(iTauvEquipmentService.getBaseData());
    }

    /**
     * 获取无人机设备列表
     *
     * @return
     */
    @PostMapping("/getEquipmentList")
    public Response getEquipmentList(@RequestBody EquipmentQuery query) {
        return iTauvEquipmentService.getEquipmentList(query);
    }

    /**
     * 实时视频列表
     * @return
     */
    @GetMapping("/live")
    @RequiresPermissions(controllerName + ":live")
    public Response liveList() {
        QueryWrapper wrapper = new QueryWrapper();
        wrapper.eq("mark", 1);
        List<TauvEquipment> list = iTauvEquipmentService.list(wrapper);
        return response.success(list);
    }
}
