package com.taauav.admin.controller;

import com.taauav.admin.controller.BaseController;
import com.taauav.admin.dto.TauvInspectPhotoDTO;
import com.taauav.common.bean.Response;
import org.apache.shiro.authz.annotation.RequiresPermissions;
import com.taauav.common.core.mps.BaseQuery;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.PutMapping;
import org.springframework.web.bind.annotation.DeleteMapping;
import org.springframework.web.bind.annotation.PathVariable;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;
import com.taauav.admin.entity.TauvInspectPhoto;
import com.taauav.admin.service.ITauvInspectPhotoService;

/**
 * 巡检正摄图Controller
 *
 * @author daixiantong
 * @date 20200518
 */
@RestController
@RequestMapping("/admin/photo")
public class TauvInspectPhotoController extends BaseController {
    @Autowired
    private ITauvInspectPhotoService tauvInspectPhotoService;

    @Autowired
    private Response response;

    /**
     * 查询巡检正摄图列表
     */
    @RequiresPermissions("admin:photo:list")
    @GetMapping("/list")
    public Response list(TauvInspectPhotoDTO inspectPhotoDTO, BaseQuery query) {
        return tauvInspectPhotoService.selectPhotoPage(inspectPhotoDTO, query);
    }

    /**
     * 获取巡检正摄图详细信息
     */
    @RequiresPermissions("admin:photo:query")
    @GetMapping(value = "/{id}")
    public Response getInfo(@PathVariable("id") Integer id) {
        return response.success(tauvInspectPhotoService.getInfo(id));
    }

    /**
     * 新增巡检正摄图
     */
    @RequiresPermissions("admin:photo:add")
    @PostMapping
    public Response add(@RequestBody TauvInspectPhoto tauvInspectPhoto) {
        return response.success(tauvInspectPhotoService.save(tauvInspectPhoto));
    }

    /**
     * 修改巡检正摄图
     */
    @RequiresPermissions("admin:photo:edit")
    @PostMapping("/edit")
    public Response edit(@RequestBody TauvInspectPhoto tauvInspectPhoto) {
        return tauvInspectPhotoService.editPhoto(tauvInspectPhoto);
    }

    /**
     * 删除巡检正摄图
     */
    @RequiresPermissions("admin:photo:remove")
    @DeleteMapping("/{ids}")
    public Response remove(@PathVariable Integer[] ids) {
        return response.success(tauvInspectPhotoService.deleteByIds(ids, new TauvInspectPhoto()));
    }
}
