package com.taauav.admin.controller;

import com.taauav.admin.controller.BaseController;
import com.taauav.admin.dto.TauvInspectQuestionDTO;
import com.taauav.common.bean.Response;
import org.apache.shiro.authz.annotation.RequiresPermissions;
import com.taauav.common.core.mps.BaseQuery;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;
import com.taauav.admin.entity.TauvInspectQuestion;
import com.taauav.admin.service.ITauvInspectQuestionService;


/**
 * 问题管理Controller
 *
 * @author daixiantong
 * @date 20200518
 */
@RestController
@RequestMapping("/admin/question")
public class TauvInspectQuestionController extends BaseController
{
    @Autowired
    private ITauvInspectQuestionService tauvInspectQuestionService;

    @Autowired
    private Response response;
    /**
     * 查询问题管理列表
     */
    @RequiresPermissions("admin:question:list")
    @GetMapping("/list")
    public Response list(TauvInspectQuestionDTO inspectQuestionDTO, BaseQuery query)
    {
        return tauvInspectQuestionService.selectQuestionPage(inspectQuestionDTO, query);
    }

    /**
     * 获取问题管理详细信息
     */
    @RequiresPermissions("admin:question:query")
    @GetMapping("/detail")
    public Response getInfo(@RequestParam("id") Integer id)
    {
        return tauvInspectQuestionService.getQuestionData(id);
    }

    /**
     * 新增问题管理
     */
    @RequiresPermissions("admin:question:add")
    @PostMapping
    public Response add(@RequestBody TauvInspectQuestion tauvInspectQuestion)
    {
        return response.success(tauvInspectQuestionService.save(tauvInspectQuestion));
    }

    /**
     * 修改问题管理
     */
    @RequiresPermissions("admin:question:edit")
    @PutMapping
    public Response edit(@RequestBody TauvInspectQuestion tauvInspectQuestion)
    {
        return response.success(tauvInspectQuestionService.updateById(tauvInspectQuestion));
    }

    /**
     * 删除问题管理
     */
    @RequiresPermissions("admin:question:remove")
    @DeleteMapping("/{ids}")
    public Response remove(@PathVariable Integer[] ids)
    {
        return response.success(tauvInspectQuestionService.deleteByIds(ids, new TauvInspectQuestion()));
    }
}
