package com.taauav.admin.controller;

import com.taauav.admin.controller.BaseController;
import com.taauav.common.bean.Response;
import org.apache.shiro.authz.annotation.RequiresPermissions;
import com.taauav.common.core.mps.BaseQuery;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.PutMapping;
import org.springframework.web.bind.annotation.DeleteMapping;
import org.springframework.web.bind.annotation.PathVariable;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;
import com.taauav.admin.entity.TauvInspectTrack;
import com.taauav.admin.service.ITauvInspectTrackService;


/**
 * 巡检飞行轨迹Controller
 *
 * @author daixiantong
 * @date 20200518
 */
@RestController
@RequestMapping("/admin/track")
public class TauvInspectTrackController extends BaseController {
    @Autowired
    private ITauvInspectTrackService tauvInspectTrackService;

    @Autowired
    private Response response;

    /**
     * 查询巡检飞行轨迹列表
     */
    @RequiresPermissions("admin:track:list")
    @GetMapping("/list")
    public Response list(TauvInspectTrack tauvInspectTrack, BaseQuery query) {
        return tauvInspectTrackService.pageData(query, tauvInspectTrack);
    }

    /**
     * 获取巡检飞行轨迹详细信息
     */
    @RequiresPermissions("admin:track:query")
    @GetMapping(value = "/{id}")
    public Response getInfo(@PathVariable("id") Integer id) {
        return response.success(tauvInspectTrackService.getInfo(id));
    }

    /**
     * 新增巡检飞行轨迹
     */
    @RequiresPermissions("admin:track:add")
    @PostMapping
    public Response add(@RequestBody TauvInspectTrack tauvInspectTrack) {
        return response.success(tauvInspectTrackService.save(tauvInspectTrack));
    }

    /**
     * 修改巡检飞行轨迹
     */
    @RequiresPermissions("admin:track:edit")
    @PutMapping
    public Response edit(@RequestBody TauvInspectTrack tauvInspectTrack) {
        return response.success(tauvInspectTrackService.updateById(tauvInspectTrack));
    }

    /**
     * 删除巡检飞行轨迹
     */
    @RequiresPermissions("admin:track:remove")
    @DeleteMapping("/{ids}")
    public Response remove(@PathVariable Integer[] ids) {
        return response.success(tauvInspectTrackService.deleteByIds(ids, new TauvInspectTrack()));
    }
}
