package com.taauav.admin.controller;

import com.taauav.admin.controller.BaseController;
import com.taauav.common.bean.Response;
import org.apache.shiro.authz.annotation.RequiresPermissions;
import com.taauav.common.core.mps.BaseQuery;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.PutMapping;
import org.springframework.web.bind.annotation.DeleteMapping;
import org.springframework.web.bind.annotation.PathVariable;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;
import com.taauav.admin.entity.TauvWaterAlarmInfo;
import com.taauav.admin.service.ITauvWaterAlarmInfoService;


/**
 * 水质报警附Controller
 *
 * @author daixiantong
 * @date 20200514
 */
@RestController
@RequestMapping("/admin/alarmInfo")
public class TauvWaterAlarmInfoController extends BaseController
{
    @Autowired
    private ITauvWaterAlarmInfoService tauvWaterAlarmInfoService;

    @Autowired
    private Response response;
    /**
     * 查询水质报警附列表
     */
    @RequiresPermissions("admin:alarmInfo:list")
    @GetMapping("/list")
    public Response list(TauvWaterAlarmInfo tauvWaterAlarmInfo, BaseQuery query)
    {
        return  tauvWaterAlarmInfoService.pageData(query,tauvWaterAlarmInfo);
    }



    /**
     * 获取水质报警附详细信息
     */
    @RequiresPermissions("admin:alarmInfo:query")
    @GetMapping(value = "/{id}")
    public Response getInfo(@PathVariable("id") Integer id)
    {
        return response.success(tauvWaterAlarmInfoService.getInfo(id));
    }

    /**
     * 新增水质报警附
     */
    @RequiresPermissions("admin:alarmInfo:add")
    @PostMapping
    public Response add(@RequestBody TauvWaterAlarmInfo tauvWaterAlarmInfo)
    {
        return response.success(tauvWaterAlarmInfoService.save(tauvWaterAlarmInfo));
    }

    /**
     * 修改水质报警附
     */
    @RequiresPermissions("admin:alarmInfo:edit")
    @PutMapping
    public Response edit(@RequestBody TauvWaterAlarmInfo tauvWaterAlarmInfo)
    {
        return response.success(tauvWaterAlarmInfoService.updateById(tauvWaterAlarmInfo));
    }

    /**
     * 删除水质报警附
     */
    @RequiresPermissions("admin:alarmInfo:remove")
    @DeleteMapping("/{ids}")
    public Response remove(@PathVariable Integer[] ids)
    {
        return response.success(tauvWaterAlarmInfoService.deleteByIds(ids, new TauvWaterAlarmInfo()));
    }
}
