package com.taauav.admin.controller;

import com.taauav.admin.entity.TauvWaterData;
import com.taauav.admin.query.TauvWaterDataQuery;
import com.taauav.admin.service.ITauvWaterDataService;
import com.taauav.common.bean.Response;
import com.taauav.common.constant.PermissionConstants;
import org.apache.shiro.authz.annotation.RequiresPermissions;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;
import org.springframework.web.multipart.MultipartFile;

import javax.annotation.Resource;

/**
 * 水质数据 控制器
 *
 * @author daixiantong
 * @date 2020-05-14
 */
@RestController
@RequestMapping("/admin/waterData")
public class TauvWaterDataController extends BaseController {
    private static final String controllerName = "admin:waterData";
    @Autowired
    private ITauvWaterDataService waterDataService;
    @Resource
    private Response response;

    /**
     * 水质数据列表
     * @param waterDataQuery
     * @return
     */
    @GetMapping("/index")
    @RequiresPermissions(controllerName + ":" + PermissionConstants.LIST_PERMISSION)
    public Response dataList(TauvWaterDataQuery waterDataQuery) {
        return waterDataService.selectPageList(waterDataQuery);
    }

    /**
     * 编辑
     * @param waterData
     * @return
     */
    @PostMapping("/edit")
    @RequiresPermissions(controllerName + ":" + PermissionConstants.EDIT_PERMISSION)
    public Response editData(TauvWaterData waterData) {
        Boolean res = waterDataService.editData(waterData);
        if (!res) {
            return response.failure("操作失敗");
        }
        return response.success("操作成功");
    }

    /**
     * 导入数据
     * @param file
     * @return
     */
    @PostMapping("/import")
    @RequiresPermissions(controllerName + ":" + PermissionConstants.IMPORT_PERMISSION)
    public Response importData(@RequestParam("file") MultipartFile file) {
        return waterDataService.importData(file);
    }

    /**
     * 下载导入模板
     * @return
     */
    @GetMapping("/template")
    public Response exportTemplate() {
        return waterDataService.getTemplate();
    }
}
