package com.taauav.admin.controller;

import com.taauav.admin.controller.BaseController;
import com.taauav.admin.dto.TauvWaterSpectrumDTO;
import com.taauav.common.bean.Response;
import org.apache.shiro.authz.annotation.RequiresPermissions;
import com.taauav.common.core.mps.BaseQuery;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;
import com.taauav.admin.entity.TauvWaterSpectrum;
import com.taauav.admin.service.ITauvWaterSpectrumService;
import org.springframework.web.multipart.MultipartFile;


/**
 * 多光谱图Controller
 *
 * @author daixiantong
 * @date 20200514
 */
@RestController
@RequestMapping("/admin/spectrum")
public class TauvWaterSpectrumController extends BaseController {
    @Autowired
    private ITauvWaterSpectrumService tauvWaterSpectrumService;

    @Autowired
    private Response response;

    /**
     * 查询多光谱图列表
     */
    @RequiresPermissions("admin:spectrum:list")
    @GetMapping("/list")
    public Response list(TauvWaterSpectrumDTO waterSpectrumDTO, BaseQuery query) {
        return tauvWaterSpectrumService.selectSpectrumPage(waterSpectrumDTO, query);
    }

    /**
     * 获取多光谱图详细信息
     */
    @RequiresPermissions("admin:spectrum:query")
    @GetMapping("/detail")
    public Response getInfo(@RequestParam("id") Integer id) {
        return response.success(tauvWaterSpectrumService.getInfo(id));
    }

    /**
     * 新增多光谱图
     */
    @RequiresPermissions("admin:spectrum:add")
    @PostMapping
    public Response add(@RequestBody TauvWaterSpectrum tauvWaterSpectrum) {
        return response.success(tauvWaterSpectrumService.save(tauvWaterSpectrum));
    }

    /**
     * 修改多光谱图
     */
    @RequiresPermissions("admin:spectrum:edit")
    @PostMapping("/edit")
    public Response edit(@RequestBody TauvWaterSpectrum tauvWaterSpectrum) {
        return tauvWaterSpectrumService.editSpectrum(tauvWaterSpectrum);
    }

    /**
     * 删除多光谱图
     */
    @RequiresPermissions("admin:spectrum:remove")
    @PostMapping("/remove")
    public Response remove(String ids) {
        return tauvWaterSpectrumService.dropSpectrum(ids);
    }

    /**
     * 上传附件
     *
     * @param file
     * @return
     */
    @PostMapping("/upload")
    public Response uploadFile(MultipartFile file) {
        return tauvWaterSpectrumService.uploadSpectrum(file);
    }
}
