package com.taauav.admin.controller;

import com.taauav.admin.entity.TauvWaterStandard;
import com.taauav.admin.query.TauvWaterStandardQuery;
import com.taauav.admin.service.ITauvWaterStandardService;
import com.taauav.common.bean.Response;
import com.taauav.common.constant.PermissionConstants;
import org.apache.shiro.authz.annotation.RequiresPermissions;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.validation.BindingResult;
import org.springframework.web.bind.annotation.*;

import javax.validation.Valid;

/**
 * 地表水环境质量标准控制器
 *
 * @author daixiantong
 * @since 2020-05-13
 */
@RestController
@RequestMapping("/admin/standard")
public class TauvWaterStandardController {
    @Autowired
    private ITauvWaterStandardService waterStandardService;
    @Autowired
    private Response response;
    private static final String controllerName = "admin:standard";

    /**
     * 标准列表
     *
     * @return
     */
    @GetMapping("/index")
    @RequiresPermissions(controllerName + ":" + PermissionConstants.LIST_PERMISSION)
    public Response index(TauvWaterStandardQuery waterStandardQuery) {
        return waterStandardService.getPageList(waterStandardQuery);
    }

    /**
     * 添加标准
     * @param waterStandard
     * @param bindingResult
     * @return
     */
    @PostMapping("/add")
    @RequiresPermissions(controllerName + ":" + PermissionConstants.ADD_PERMISSION)
    public Response add(@Valid @RequestBody TauvWaterStandard waterStandard, BindingResult bindingResult) {
        if (bindingResult.hasErrors()) {
            String msg = bindingResult.getFieldError().getDefaultMessage();
            return response.failure(msg);
        }
        return waterStandardService.editStandard(waterStandard);
    }

    /**
     * 编辑标准
     * @param waterStandard
     * @param bindingResult
     * @return
     */
    @PostMapping("/edit")
    @RequiresPermissions(controllerName + ":" + PermissionConstants.EDIT_PERMISSION)
    public Response edit(@Valid @RequestBody TauvWaterStandard waterStandard, BindingResult bindingResult) {
        if (bindingResult.hasErrors()) {
            String msg = bindingResult.getFieldError().getDefaultMessage();
            return response.failure(msg);
        }
        return waterStandardService.editStandard(waterStandard);
    }

    /**
     * 删除
     * @param id
     * @return
     */
    @PostMapping("/drop")
    @RequiresPermissions(controllerName + ":" + PermissionConstants.REMOVE_PERMISSION)
    public Response drop(Integer id) {
        return waterStandardService.dropStandard(id);
    }
}
