package com.taauav.admin.controller;

import com.taauav.admin.controller.BaseController;
import com.taauav.admin.dto.TauvWaterTrendDTO;
import com.taauav.common.bean.Response;
import com.taauav.common.util.DateUtil;
import com.taauav.common.util.ShiroUtils;
import org.apache.shiro.authz.annotation.RequiresPermissions;
import com.taauav.common.core.mps.BaseQuery;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;
import com.taauav.admin.entity.TauvWaterTrend;
import com.taauav.admin.service.ITauvWaterTrendService;
import org.springframework.web.multipart.MultipartFile;

/**
 * 水环境趋势模拟Controller
 *
 * @author daixiantong
 * @date 20200515
 */
@RestController
@RequestMapping("/admin/trend")
public class TauvWaterTrendController extends BaseController {
    @Autowired
    private ITauvWaterTrendService tauvWaterTrendService;

    @Autowired
    private Response response;

    /**
     * 查询水环境趋势模拟列表
     */
//    @RequiresPermissions("admin:trend:list")
    @GetMapping("/list")
    public Response list(TauvWaterTrendDTO tauvWaterTrendDTO, BaseQuery query) {
        return tauvWaterTrendService.selectTrendPage(query, tauvWaterTrendDTO);
    }

    /**
     * 获取水环境趋势模拟详细信息
     */
    @RequiresPermissions("admin:trend:query")
    @GetMapping("/detail")
    public Response getInfo(@RequestParam("id") Integer id) {
        return tauvWaterTrendService.getTrendList(id);
    }

    /**
     * 新增水环境趋势模拟
     */
    @RequiresPermissions("admin:trend:add")
    @PostMapping
    public Response add(@RequestBody TauvWaterTrend tauvWaterTrend) {
        return response.success(tauvWaterTrendService.save(tauvWaterTrend));
    }

    /**
     * 修改水环境趋势模拟
     */
    @RequiresPermissions("admin:trend:edit")
    @PostMapping("/edit")
    public Response edit(@RequestBody TauvWaterTrend tauvWaterTrend) {
        tauvWaterTrend.setUpdateTime(DateUtil.now());
        tauvWaterTrend.setUpdateUser(ShiroUtils.getAdminId());
        return response.success(tauvWaterTrendService.updateById(tauvWaterTrend));
    }

    /**
     * 删除水环境趋势模拟
     */
    @RequiresPermissions("admin:trend:remove")
    @DeleteMapping("/{ids}")
    public Response remove(@PathVariable Integer[] ids) {
        return response.success(tauvWaterTrendService.deleteByIds(ids, new TauvWaterTrend()));
    }

    /**
     * 导入趋势数据
     *
     * @param file
     * @return
     */
    @PostMapping("/import")
    public Response importExcel(@RequestParam("file") MultipartFile file) {
        return tauvWaterTrendService.importTrend(file);
    }

    /**
     * 下载导入模板
     *
     * @return
     */
    @GetMapping("/template")
    public Response exportTemplate() {
        return tauvWaterTrendService.getTemplate();
    }
}
