package com.taauav.admin.controller;

import com.baomidou.mybatisplus.core.metadata.IPage;
import com.google.common.base.Joiner;
import com.taauav.admin.controller.BaseController;
import com.taauav.common.bean.Response;
import com.taauav.common.util.DateUtil;
import com.taauav.common.util.FunctionUtils;
import org.apache.shiro.authz.annotation.RequiresPermissions;
import com.taauav.common.core.mps.BaseQuery;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;
import com.taauav.admin.entity.TauvWaterWarn;
import com.taauav.admin.service.ITauvWaterWarnService;

import java.util.ArrayList;
import java.util.List;


/**
 * 水质预警Controller
 *
 * @author daixiantong
 * @date 20200514
 */
@RestController
@RequestMapping("/admin/warn")
public class TauvWaterWarnController extends BaseController {
    @Autowired
    private ITauvWaterWarnService tauvWaterWarnService;

    @Autowired
    private Response response;

    /**
     * 查询水质预警列表
     */
    @RequiresPermissions("admin:warn:list")
    @GetMapping("/list")
    public Response list(TauvWaterWarn tauvWaterWarn, BaseQuery query) {
        IPage<TauvWaterWarn> list = tauvWaterWarnService.pageData(query, tauvWaterWarn);
        if (list.getRecords().size() > 0) {
            for (TauvWaterWarn data : list.getRecords()) {
                data.setWarnTypeText(FunctionUtils.getArrayText(data.getWarnType().toString(), data.typeList()));
                data.setWarnLevelText(FunctionUtils.getArrayText(data.getWarnLevel().toString(), data.levelList()));
                data.setStatusText(FunctionUtils.getArrayText(data.getStatus().toString(), data.statusList()));
            }
        }
        return response.success(list);
    }

    /**
     * 获取水质预警详细信息
     */
    @RequiresPermissions("admin:warn:query")
    @GetMapping("detail")
    public Response getInfo(@RequestParam("id") Integer id) {
        TauvWaterWarn waterWarn = tauvWaterWarnService.getInfo(id);
        if (waterWarn == null) {
            return response.failure("数据不存在");
        }
        waterWarn.setWarnTypeText(FunctionUtils.getArrayText(waterWarn.getWarnType().toString(), waterWarn.typeList()));
        waterWarn.setWarnLevelText(FunctionUtils.getArrayText(waterWarn.getWarnLevel().toString(), waterWarn.levelList()));
        waterWarn.setStatusText(FunctionUtils.getArrayText(waterWarn.getStatus().toString(), waterWarn.statusList()));
        return response.success(waterWarn);
    }

    /**
     * 新增水质预警
     */
    @RequiresPermissions("admin:warn:add")
    @PostMapping("/add")
    public Response add(@RequestBody TauvWaterWarn tauvWaterWarn) {
        return response.success(tauvWaterWarnService.save(tauvWaterWarn));
    }

    /**
     * 修改水质预警
     */
    @RequiresPermissions("admin:warn:edit")
    @PostMapping("/edit")
    public Response edit(@RequestBody TauvWaterWarn tauvWaterWarn) {
        tauvWaterWarn.setWarnTime(DateUtil.now());
        tauvWaterWarn.setStatus(2);
        return response.success(tauvWaterWarnService.updateById(tauvWaterWarn));
    }

    /**
     * 删除水质预警
     */
    @RequiresPermissions("admin:warn:remove")
    @PostMapping("/remove")
    public Response remove(String ids) {
        return tauvWaterWarnService.dropWarn(ids);
    }

    /**
     * 取消预警
     *
     * @param ids
     * @return
     */
    @PostMapping("/cancel")
    @RequiresPermissions("admin:warn:cancel")
    public Response cancelWarn(String ids) {
        return tauvWaterWarnService.cancelWarn(ids);
    }
}
