package com.taauav.admin.entity;

import com.baomidou.mybatisplus.annotation.IdType;
import com.baomidou.mybatisplus.annotation.TableField;
import com.baomidou.mybatisplus.annotation.TableId;
import com.baomidou.mybatisplus.annotation.TableName;
import com.fasterxml.jackson.annotation.JsonFormat;
import com.taauav.common.domain.Entity;
import lombok.Data;
import lombok.EqualsAndHashCode;
import org.hibernate.validator.constraints.Length;
import org.springframework.format.annotation.DateTimeFormat;

import javax.validation.constraints.NotEmpty;
import javax.validation.constraints.NotNull;
import javax.validation.constraints.Pattern;
import java.time.LocalDateTime;
import java.util.Date;
import java.util.List;
import java.util.Map;

/**
 * <p>
 * 管理员表
 * </p>
 *
 * @author dyg
 * @since 2019-11-04
 */
@Data
@EqualsAndHashCode(callSuper = false)
@TableName("sys_admin")
public class SysAdmin extends Entity {

    private static final long serialVersionUID = 1L;

    /**
     * 编号
     */
    @TableId(value = "id", type = IdType.AUTO)
    private Integer id;
    /**
     * 姓名
     */
    @NotEmpty(message = "姓名不能为空")
    @Length(min = 2, max = 10, message = "姓名长度为2-10")
    private String realname;

    /**
     * 用户名
     */
    @NotEmpty(message = "用户名不能为空")
    @Length(min = 2, max = 20, message = "用户名长度为2-20个字符")
    private String username;

    /**
     * 密码
     */
    private String password;

    /**
     * 管理员拥有的规则id， 多个规则","隔开'
     */
    private String rules;

    /**
     * 头像
     */
    private String avatar;

    /**
     * 公司名称
     */
    @NotEmpty(message = "公司名称不能为空")
    @Length(min = 2, max = 20, message = "公司名称长度为2-20个字符")
    private String company;

    /**
     * 职务
     */
    @Length(max = 10, message = "职务长度不能超过10个字符")
    private String duty;

    /**
     * 角色编号(子级)
     */
    @TableField("auth_group")
    @NotNull(message = "请选择角色")
    private String authGroup;

    /**
     * 联系方式（手机号码）
     */
    @NotEmpty(message = "手机号不能为空")
    @Pattern(regexp = "^1[3-9][0-9]{9}", message = "手机号格式不正确")
    private String mobile;

    @Override
    public String toString() {
        return "SysAdmin{" +
                "id=" + id +
                ", realname='" + realname + '\'' +
                ", username='" + username + '\'' +
                ", password='" + password + '\'' +
                ", rules='" + rules + '\'' +
                ", avatar='" + avatar + '\'' +
                ", company='" + company + '\'' +
                ", duty='" + duty + '\'' +
                ", authGroup='" + authGroup + '\'' +
                ", mobile='" + mobile + '\'' +
                ", loginNum=" + loginNum +
                ", loginFailure=" + loginFailure +
                ", lastIp=" + lastIp +
                ", lastTime=" + lastTime +
                ", status=" + this.getStatus() +
                ", sort=" + sort +
                ", createUser=" + createUser +
                ", updateUser=" + updateUser +
                ", createTime=" + createTime +
                ", updateTime=" + updateTime +
                ", mark=" + mark +
                '}';
    }

    /**
     * 区属城市ID
     */
    @TableField("city_ids")
    private String cityIds;

    /**
     * 登录次数
     */
    @TableField("login_num")
    private Integer loginNum;

    /**
     * 失败次数
     */
    @TableField("login_failure")
    private Integer loginFailure;

    /**
     * 上次登录IP
     */
    @TableField("last_ip")
    private Long lastIp;

    /**
     * 上次登录时间
     */
    @TableField("last_time")
    private Integer lastTime;


    /**
     * 排序
     */
    private Integer sort;

    /**
     * 创建人
     */
    @TableField("create_user")
    private Integer createUser;

    /**
     * 更新人
     */
    @TableField("update_user")
    private Integer updateUser;

    /**
     * 创建时间
     */
    @TableField("create_time")
    @DateTimeFormat(pattern = "yyyy-MM-dd HH:mm:ss")
    @JsonFormat(pattern = "yyyy-MM-dd HH:mm:ss", timezone = "GMT+8")
    private Date createTime;

    /**
     * 更新时间
     */
    @TableField("update_time")
    @DateTimeFormat(pattern = "yyyy-MM-dd HH:mm:ss")
    @JsonFormat(pattern = "yyyy-MM-dd HH:mm:ss", timezone = "GMT+8")
    private Date updateTime;

    /**
     * 有效标记
     */
    private Integer mark;

    /**
     * 角色描述
     */
    @TableField(exist = false)
    private String authGroupText;

    /**
     * 状态描述
     */
    @TableField(exist = false)
    private String statusText;

    /**
     * 区域城市
     */
    @TableField(exist = false)
    private List<Map<String, Object>> cityList;
}
