package com.taauav.admin.entity;

import java.math.BigDecimal;


import java.io.Serializable;
import java.math.BigInteger;
import java.util.Date;

import com.fasterxml.jackson.annotation.JsonFormat;
import com.taauav.common.domain.Entity;
import lombok.Data;
import lombok.EqualsAndHashCode;
import lombok.experimental.Accessors;
import org.springframework.format.annotation.DateTimeFormat;

import javax.validation.constraints.NotEmpty;

/**
 * <p>
 * 巡检任务河道表
 * </p>
 *
 * @author dyg
 * @since 2019-11-14
 */
@Data
@EqualsAndHashCode(callSuper = false)
@Accessors(chain = true)
public class TauvInspectDriver extends Entity {

    private static final long serialVersionUID = 1L;

    /**
     * 巡检任务ID
     */
    private Integer inspectId;

    /**
     * 巡检任务来源：1平台发布 2客户发起
     */
    private Integer source;

    /**
     * 行号(以10的倍数递增)
     */
    private Integer num;

    /**
     * 巡检任务单号对应的河流子编号
     */
    private String inspectNo;

    /**
     * 河道ID
     */
    @NotEmpty(message = "河道编号不能为空")
    private Integer driverId;

    /**
     * 河流名称
     */
    @NotEmpty(message = "河流名称不为空")
    private String driverName;

    /**
     * 河流区域ID
     */
    @NotEmpty(message = "河流区域不能为空")
    private BigInteger driverArea;

    /**
     * 河流长度(单位：KM)
     */
    @NotEmpty(message = "河流长度不能为空")
    private BigDecimal driverLength;

    /**
     * 河流起点
     */
    @NotEmpty(message = "河流起点不能为空")
    private String driverStart;

    /**
     * 河流终点
     */
    @NotEmpty(message = "河流终点不能为空")
    private String driverEnd;

    /**
     * 河流图片
     */
    private String driverImage;

    /**
     * 航线文件名
     */
    private String driverFilename;

    /**
     * 航线文件
     */
    private String driverFile;

    /**
     * 河流备注
     */
    private String driverNote;

    /**
     * 飞行设备ID
     */
    private String equipmentId;

    /**
     * 挂载设备ID
     */
    private String mountequipmentId;

    /**
     * 飞手ID
     */
    private Integer flightHand;

    /**
     * 巡检任务飞手
     */
    private String inspectHand;

    /**
     * 执行时间
     */
    @DateTimeFormat(pattern = "yyyy-MM-dd")
    @JsonFormat(pattern = "yyyy-MM-dd", timezone = "GMT+8")
    private Date executionTime;

    /**
     * 天气
     */
    private String weather;

    /**
     * 风向
     */
    private String wind;

    /**
     * 温度
     */
    private String temperature;

    /**
     * 飞行速度
     */
    private BigDecimal flightSpeed;

    /**
     * 飞行时长
     */
    private BigDecimal flightTime;

    /**
     * 飞行高度
     */
    private BigDecimal flightHeight;

    /**
     * 备注
     */
    private String remark;

    /**
     * 复核人
     */
    private Integer reviewUser;
    /**
     * 复核时间
     */
    @DateTimeFormat(pattern = "yyyy-MM-dd HH:mm:ss")
    @JsonFormat(pattern = "yyyy-MM-dd HH:mm:ss", timezone = "GMT+8")
    private Date reviewTime;

}
