package com.taauav.admin.service;

import com.baomidou.mybatisplus.extension.service.IService;
import com.taauav.admin.vo.SysCityListVo;
import com.taauav.common.bean.Response;
import com.taauav.admin.entity.SysCity;

import java.math.BigInteger;
import java.util.List;
import java.util.Map;

/**
 * <p>
 * 城市表 服务类
 * </p>
 *
 * @author dyg
 * @since 2019-11-13
 */
public interface ISysCityService extends IService<SysCity> {

    /**
     * 获取机构列表
     *
     * @return
     */
    Response getList();

    /**
     * 添加机构
     *
     * @param entity 实体对象
     * @return
     */
    Response add(SysCity entity);

    /**
     * 获取机构详情
     *
     * @param id 机构ID
     * @return
     */
    Response info(BigInteger id);

    /**
     * 编辑机构
     *
     * @param entity 实体对象
     * @return
     */
    Response edit(SysCity entity);

    /**
     * 删除机构
     *
     * @param id 机构ID
     * @return
     */
    Response drop(BigInteger id);

    /**
     * 设置状态
     *
     * @param entity 实体对象
     * @return
     */
    Response setStatus(SysCity entity);

    /**
     * 根据父级编号获取数据
     *
     * @param pid 320100  南京编号
     * @return
     */
    List<Map<String, Object>> getCityListByPid(Integer pid);

    /**
     * 获取城市列表
     *
     * @param pid
     * @param type
     * @return
     */
    List<SysCity> getCityList(BigInteger pid, String type);

    /**
     * 根据父级编号和用户信息获取城市列表
     *
     * @param pid
     * @return
     */
    List<Map<String, Object>> getCityListByPidAndAdmin(Integer pid);

    /**
     * 根据id获取城市数据
     *
     * @param id
     * @return
     */
    SysCity getInfoById(BigInteger id);

    /**
     * 获取城市各辖区河道信息
     *
     * @param cityId
     * @return
     */
    Response getCityDriverList(Integer cityId);

    /**
     * 根据城市ID获取城市名称
     *
     * @param cityId
     * @return
     */
    String getCityName(BigInteger cityId);

    /**
     * 获取父级城市数据
     *
     * @param cityId
     * @return
     */
    List<SysCity> getParentCityList(BigInteger cityId);

    /**
     * 获取子级城市
     *
     * @param pid 上级ID
     * @return
     */
    List<SysCityListVo> getChildCityList(BigInteger pid);

    /**
     * 获取子级城市ID
     *
     * @param pid 上级城市
     * @return
     */
    List<BigInteger> getChildCityIds(BigInteger pid);
}
