package com.taauav.admin.service.impl;

import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.taauav.common.bean.Response;
import com.taauav.admin.entity.SysAuthGroupAccess;
import com.taauav.admin.mapper.SysAuthGroupAccessMapper;
import com.taauav.admin.service.ISysAuthGroupAccessService;
import com.taauav.common.service.impl.BaseServiceImpl;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.util.StringUtils;

import java.util.*;

/**
 * <p>
 * 用户组明细表 服务实现类
 * </p>
 *
 * @author dyg
 * @since 2019-11-04
 */
@Service
public class SysAuthGroupAccessServiceImpl extends BaseServiceImpl<SysAuthGroupAccessMapper, SysAuthGroupAccess> implements ISysAuthGroupAccessService {
    @Autowired
    private Response response;
    @Autowired
    private SysAuthGroupAccessMapper sysAuthGroupAccessMapper;

    @Override
    public List<SysAuthGroupAccess> getListByGroupid(Integer groupId) {
        QueryWrapper<SysAuthGroupAccess> queryWrapper = new QueryWrapper<>();
        queryWrapper.eq("group_id", groupId);
        return sysAuthGroupAccessMapper.selectList(queryWrapper);
    }

    @Override
    public Map<String, String> add(Integer adminId, String groupIds) {
        Map<String, String> result = new HashMap<>();
        result.put("code", "500");
        if (StringUtils.isEmpty(adminId)) {
            result.put("msg", "请选择用户");
            return result;
        }
        if (StringUtils.isEmpty(groupIds)) {
            result.put("msg", "请选择角色");
            return result;
        }
        String[] groupIdList = groupIds.split(",");
        Set<String> requestGroupIds = new HashSet<String>(Arrays.asList(groupIdList));

        Map<String, String> hasGroupMap = new HashMap<>();
        List<SysAuthGroupAccess> groupAccessList = getListByAdmin(adminId);
        if (groupAccessList != null) {
            for (SysAuthGroupAccess sysAuthGroupAccess : groupAccessList) {
                String val = sysAuthGroupAccess.getGroupId().toString();
                hasGroupMap.put(val, val);
            }
        }
        //判断权限是否已存在，不存在时添加
        for (String groupId : groupIdList) {
            if (hasGroupMap.containsKey(groupId)) {
                continue;
            } else {
                SysAuthGroupAccess insertData = new SysAuthGroupAccess();
                insertData.setUid(adminId);
                insertData.setGroupId(Integer.valueOf(groupId));
                sysAuthGroupAccessMapper.insert(insertData);
            }
        }
        //删除已存在多余角色权限
        for (String hGroup : hasGroupMap.values()) {
            if (!requestGroupIds.contains(hGroup)) {
                QueryWrapper<SysAuthGroupAccess> delSet = new QueryWrapper<>();
                delSet.eq("uid", adminId);
                delSet.eq("group_id", Integer.valueOf(hGroup));
                sysAuthGroupAccessMapper.delete(delSet);
            }
        }
        result.put("code", "200");
        result.put("msg", "ok");
        return result;
    }

    @Override
    public List<SysAuthGroupAccess> getListByAdmin(Integer adminId) {
        QueryWrapper<SysAuthGroupAccess> queryWrapper = new QueryWrapper<>();
        queryWrapper.eq("uid", adminId);
        return sysAuthGroupAccessMapper.selectList(queryWrapper);
    }

}
