package com.taauav.admin.service.impl;

import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.taauav.admin.vo.SysCityListVo;
import com.taauav.common.bean.Response;
import com.taauav.admin.entity.SysAdmin;
import com.taauav.admin.entity.SysCity;
import com.taauav.admin.mapper.SysAdminMapper;
import com.taauav.admin.mapper.SysCityMapper;
import com.taauav.admin.mapper.TauvDriverMapper;
import com.taauav.admin.service.ISysCityService;
import com.taauav.common.util.DateUtil;
import com.taauav.common.util.RedisUtils;
import com.taauav.common.util.ShiroUtils;
import com.taauav.common.util.StringUtils;
import org.springframework.beans.BeanUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import javax.annotation.Resource;
import java.math.BigInteger;
import java.util.*;

/**
 * <p>
 * 城市表 服务实现类
 * </p>
 *
 * @author dyg
 * @since 2019-11-13
 */
@Service
public class SysCityServiceImpl extends ServiceImpl<SysCityMapper, SysCity> implements ISysCityService {
    @Autowired
    private SysCityMapper sysCityMapper;
    @Autowired
    private SysAdminMapper adminMapper;
    @Autowired
    private TauvDriverMapper tauvDriverMapper;
    @Resource
    private Response response;
    @Autowired
    private RedisUtils redisUtils;


    /**
     * 获取机构列表
     *
     * @return
     */
    @Override
    public Response getList() {
        // 查询条件
        QueryWrapper<SysCity> queryWrapper = new QueryWrapper<>();
        queryWrapper.eq("id", BigInteger.valueOf(320117));
        queryWrapper.eq("mark", 1);
        queryWrapper.orderByAsc("sort");

        // 查询条件
        SysCity cityInfo = sysCityMapper.selectById(BigInteger.valueOf(320117));
        SysCityListVo cityListVo = new SysCityListVo();
        // 拷贝属性
        BeanUtils.copyProperties(cityInfo, cityListVo);
        // 获取子级
        List<SysCityListVo> cityList = this.getChildCityList(BigInteger.valueOf(320117));
        // 添加子级
        cityListVo.setChildrenList(cityList);

        // 返回结果
        List<SysCityListVo> cityListVoList = new ArrayList<>();
        cityListVoList.add(cityListVo);
        return response.success(cityListVoList);
    }

    /**
     * 添加机构
     *
     * @param entity 实体对象
     * @return
     */
    @Override
    public Response add(SysCity entity) {
        if (entity == null) {
            return response.failure("实体对象不能为空");
        }
        if (entity.getPid() == null) {
            entity.setLevel(1);
        } else {
            SysCity agency = sysCityMapper.selectById(entity.getPid());
            if (entity != null) {
                entity.setLevel(agency.getLevel() + 1);
            }
        }
        entity.setCreateTime(DateUtil.now());
        entity.setCreateUser(ShiroUtils.getAdminId());
        int result = sysCityMapper.insert(entity);
        if (result == 0) {
            return response.failure("添加失败");
        }
        return response.success("添加成功");
    }

    /**
     * 获取机构详情
     *
     * @param id 机构ID
     * @return
     */
    @Override
    public Response info(BigInteger id) {
        if (id == null) {
            return response.failure("机构ID不存在");
        }
        SysCity entity = this.getById(id);
        if (entity == null) {
            return response.failure("机构信息不存在");
        }
        return response.success(entity);
    }

    /**
     * 编辑机构
     *
     * @param entity 实体对象
     * @return
     */
    @Override
    public Response edit(SysCity entity) {
        if (entity == null || entity.getId() == null) {
            return response.failure("实体对象不能为空");
        }
        // 同名不给添加
        QueryWrapper<SysCity> queryWrapper = new QueryWrapper<>();
        queryWrapper.eq("name", entity.getName());
        queryWrapper.eq("level", entity.getLevel());
        queryWrapper.eq("mark", 1);
        SysCity cityInfo = sysCityMapper.selectOne(queryWrapper);
        if (cityInfo != null) {
            return response.failure("同级已存在该组织名称");
        }
        entity.setUpdateUser(ShiroUtils.getAdminId());
        entity.setUpdateTime(DateUtil.now());
        boolean result = this.updateById(entity);
        if (!result) {
            return response.failure("编辑失败");
        }
        return response.success("编辑成功");
    }

    /**
     * 删除机构
     *
     * @param id 机构ID
     * @return
     */
    @Override
    public Response drop(BigInteger id) {
        if (id == null) {
            return response.failure("机构ID不存在");
        }
        if (id == BigInteger.valueOf(Long.valueOf(320117))) {
            return response.failure("根级无法删除");
        }
        SysCity entity = sysCityMapper.selectById(id);
        if (entity == null) {
            return response.failure("机构信息不存在");
        }
        // 校验是否存在子级
        QueryWrapper<SysCity> queryWrapper = new QueryWrapper<>();
        queryWrapper.eq("pid", entity.getId());
        queryWrapper.eq("mark", 1);
        Integer count = sysCityMapper.selectCount(queryWrapper);
        if (count > 0) {
            return response.failure("存在子级，无法删除");
        }
        entity.setUpdateUser(ShiroUtils.getAdminId());
        entity.setUpdateTime(DateUtil.now());
        entity.setMark(0);
        boolean result = this.updateById(entity);
        if (!result) {
            return response.failure("删除失败");
        }
        return response.success("删除成功");
    }

    /**
     * 设置状态
     *
     * @param entity 实体对象
     * @return
     */
    @Override
    public Response setStatus(SysCity entity) {
        if (entity.getId() == null) {
            return response.failure("设备ID不存在");
        }
        if (entity.getStatus() == null || entity.getStatus() <= 0) {
            return response.failure("状态不存在");
        }
        int result = sysCityMapper.updateById(entity);
        if (result == 0) {
            return response.failure("设置失败");
        }
        return response.success("设置成功");
    }

    /**
     * 根据上级编号获取数据
     *
     * @param pid
     * @return
     */
    @Override
    public List<Map<String, Object>> getCityListByPid(Integer pid) {
        QueryWrapper<SysCity> queryWrapper = new QueryWrapper<>();
        queryWrapper.eq("pid", pid);
        queryWrapper.eq("mark", 1);
        queryWrapper.select("id,pid,name,longitude,latitude");
        List<Map<String, Object>> list = sysCityMapper.selectMaps(queryWrapper);
        return list;
    }

    /**
     * 获取城市列表
     *
     * @param pid 上级ID
     * @param type
     * @return
     */
    @Override
    public List<SysCity> getCityList(BigInteger pid, String type) {
        String key = "city_all_list";
        List<SysCity> cityList = null;
        cityList = (List<SysCity>) redisUtils.get(key);
        if (StringUtils.isNull(cityList) || cityList.isEmpty()) {
            cityList = getAllCityList(pid, type);
            redisUtils.set(key, cityList);
        }
        return cityList;
    }

    /**
     * 根据父级id和用户城市权限获取城市列表
     * 备注：待完善
     *
     * @param pid
     * @return
     */
    @Override
    public List<Map<String, Object>> getCityListByPidAndAdmin(Integer pid) {
        Integer adminId = ShiroUtils.getAdminId();
        SysAdmin admin = adminMapper.selectById(adminId);
        if (!"".equals(admin.getCityIds())) {
            QueryWrapper<SysCity> wrapper = new QueryWrapper<>();
            if (!StringUtils.isEmpty(admin.getCityIds())) {
                String[] ids = admin.getCityIds().split(",");
                wrapper.in("id", ids);
            }
            wrapper.eq("pid", pid);
            wrapper.eq("mark", 1);
            wrapper.select("id,pid,name,longitude,latitude");
            List<Map<String, Object>> list = sysCityMapper.selectMaps(wrapper);
            return list;
        } else {
            return null;
        }
    }

    @Override
    public SysCity getInfoById(BigInteger id) {
        return sysCityMapper.selectById(id);
    }

    /**
     * 获取城市各辖区河道信息
     *
     * @param cityId
     * @return
     */
    @Override
    public Response getCityDriverList(Integer cityId) {
        List<Map<String, Object>> cityList = getCityListByPid(cityId);
        if (cityList != null) {
            for (Map<String, Object> city : cityList) {
                QueryWrapper wrapper = new QueryWrapper();
                wrapper.eq("mark", 1);
                wrapper.eq("status", 1);
                wrapper.eq("driver_area", city.get("id"));
                Integer num = tauvDriverMapper.selectCount(wrapper);
                city.put("count", num);
            }
        }
        return response.success(cityList);
    }

    /**
     * 根据城市ID获取城市名称
     *
     * @param cityId 城市ID
     * @return
     */
    @Override
    public String getCityName(BigInteger cityId) {
        List<String> nameList = new ArrayList<>();
        while (cityId != null && cityId.compareTo(BigInteger.valueOf(320117)) >= 0) {
            SysCity cateInfo = sysCityMapper.selectById(cityId);
            if (cateInfo != null) {
                nameList.add(cateInfo.getName());
                cityId = cateInfo.getPid();
            } else {
                cityId = null;
            }
        }
        // 使用集合工具实现数组翻转
        Collections.reverse(nameList);
        return org.apache.commons.lang3.StringUtils.join(nameList.toArray(), ">>");
    }

    @Override
    public List<SysCity> getParentCityList(BigInteger cityId) {
        SysCity city = getInfoById(cityId);
        if (city == null) {
            return null;
        }
        List<SysCity> parentList = new ArrayList<>();
        if (city.getPid() != null && city.getPid().compareTo(BigInteger.ZERO) > 0) {
            SysCity parent = getInfoById(city.getPid());
            if (parent != null) {
                parentList.add(parent);
                if (parent.getPid() != null && parent.getPid().compareTo(BigInteger.ZERO) > 0) {
                    SysCity parentCity = getInfoById(parent.getPid());
                    if (parentCity != null) {
                        parentList.add(parentCity);
                    }
                }
            }
        }
        return parentList;
    }

    /**
     * 获取子级城市ID
     *
     * @param pid 上级城市
     * @return
     */
    @Override
    public List<BigInteger> getChildCityIds(BigInteger pid) {
        QueryWrapper<SysCity> queryWrapper = new QueryWrapper<>();
        queryWrapper.eq("pid", pid);
        queryWrapper.eq("mark", 1);
        List<BigInteger> cityIdsList = new ArrayList<>();
        List<SysCity> cityList = sysCityMapper.selectList(queryWrapper);
        if (!cityList.isEmpty()) {
            cityList.forEach(item -> {
                // 获取子级
                if (item.getLevel() <= 4) {
                    List<BigInteger> itemList = this.getChildCityIds(item.getId());
                    if (!itemList.isEmpty()) {
                        itemList.forEach(subItem -> {
                            cityIdsList.add(subItem);
                        });
                    }
                }
                cityIdsList.add(item.getId());
            });
        }
        return cityIdsList;
    }

    /**
     * 获取子级城市列表
     *
     * @param pid 上级ID
     * @return
     */
    @Override
    public List<SysCityListVo> getChildCityList(BigInteger pid) {
        QueryWrapper<SysCity> queryWrapper = new QueryWrapper<>();
        queryWrapper.eq("pid", pid);
        queryWrapper.eq("mark", 1);
        queryWrapper.orderByAsc("sort");
        List<SysCity> cityList = sysCityMapper.selectList(queryWrapper);
        List<SysCityListVo> cityListVoList = new ArrayList<>();
        if (!cityList.isEmpty()) {
            cityList.forEach(item -> {
                SysCityListVo cityListVo = new SysCityListVo();
                // 拷贝属性
                BeanUtils.copyProperties(item, cityListVo);
                // 判断是否有子级
                List<SysCityListVo> childrenList = this.getChildCityList(item.getId());
                if (!childrenList.isEmpty()) {
                    cityListVo.setChildrenList(childrenList);
                    cityListVo.setHasChildren(true);
                } else {
                    cityListVo.setHasChildren(false);
                }
                cityListVoList.add(cityListVo);
            });
        }
        return cityListVoList;
    }

    /**
     * 获取所有城市数据
     * @param pid
     * @param type
     * @return
     */
    private List<SysCity> getAllCityList(BigInteger pid, String type) {
        List<SysCity> cityList = new ArrayList<>();
        if (pid.toString().equals("3201")) {
            QueryWrapper<SysCity> queryWrapper2 = new QueryWrapper<>();
            queryWrapper2.eq("id", pid);
            queryWrapper2.eq("mark", 1);
            queryWrapper2.select("id,pid,name,longitude,latitude");
            SysCity city = sysCityMapper.selectOne(queryWrapper2);
            cityList.add(city);
        }
        QueryWrapper<SysCity> queryWrapper = new QueryWrapper<>();
        queryWrapper.eq("pid", pid);
        queryWrapper.eq("mark", 1);
        if (StringUtils.isEmpty(type) || !type.equals("country")) {
            queryWrapper.ne("level", 5);
        }
        queryWrapper.select("id,pid,name,longitude,latitude");
        List<SysCity> list = sysCityMapper.selectList(queryWrapper);
        if (!list.isEmpty()) {
            // 获取子级
            list.forEach(item -> {
//                List<SysCity> children = this.getCityList(item.getId(), type);
//                if (!children.isEmpty()) {
//                    for (SysCity child : children) {
//                        cityList.add(child);
//                    }
//                }
                cityList.add(item);
            });
        }
        return cityList;
    }
}
