package com.taauav.admin.service.impl;

import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.google.common.base.Joiner;
import com.taauav.admin.dto.TauvInspectDriverAssignTaskDTO;
import com.taauav.admin.vo.TauvEquipmentListVo;
import com.taauav.admin.vo.TauvMountEquipmentListVo;
import com.taauav.admin.vo.inspectdriver.TauvInspectDriverInfoVo;
import com.taauav.admin.vo.inspectdriver.TauvInspectDriverListVo;
import com.taauav.common.bean.Response;
import com.taauav.admin.dto.TauvInspectDriverDTO;
import com.taauav.admin.entity.*;
import com.taauav.admin.mapper.TauvInspectDriverMapper;
import com.taauav.admin.mapper.TauvInspectMapper;
import com.taauav.admin.service.*;
import com.taauav.common.service.impl.BaseServiceImpl;
import com.taauav.common.util.DateUtil;
import com.taauav.common.util.FileUtil;
import com.taauav.common.util.FunctionUtils;
import com.taauav.common.util.ShiroUtils;
import com.taauav.admin.validate.ExecuteTask;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.BeanUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.stereotype.Service;
import org.springframework.util.StringUtils;

import java.io.File;
import java.math.BigDecimal;
import java.util.*;

/**
 * <p>
 * 巡检任务河道表 服务实现类
 * </p>
 *
 * @author dyg
 * @since 2019-11-14
 */
@Service
@Slf4j
public class TauvInspectDriverServiceImpl extends BaseServiceImpl<TauvInspectDriverMapper, TauvInspectDriver> implements ITauvInspectDriverService {
    @Value("${spring.pageSize}")
    private Integer pageSize;
    @Value("${server.UPLOAD_URL}")
    private String uploadUrl;
    @Value("${file.uploadFolder}")
    private String uploadFolder;
    @Autowired
    private TauvInspectDriverMapper inspectDriverMapper;
    @Autowired
    private ITauvDriverService iTauvDriverService;
    @Autowired
    private ISysAdminService iSysAdminService;
    @Autowired
    private ISysCityService iSysCityService;
    @Autowired
    private Response response;
    @Autowired
    private ITauvReportService iTauvReportService;
    //为了避免相互依赖 这里直接引用Inspect的DAO接口类，没有引用对应的业务service接口
    @Autowired
    private TauvInspectMapper inspectMapper;

    @Autowired
    private ITauvEquipmentService equipmentService;

    @Autowired
    private ITauvMountEquipmentService mountEquipmentService;

    private QueryWrapper wrapper;

    /**
     * 添加巡检任务
     *
     * @param inspect
     */
    @Override
    public String batchAddData(TauvInspect inspect) {
        List<TauvInspectDriver> inspectDriver = inspect.getInspectDriverList();
        Integer inspectId = inspect.getId();
        String errorMsg = "";
        if (inspectDriver != null && !"".equals(inspectDriver) && inspectDriver.size() > 0) {
            int i = 1;
            for (TauvInspectDriver driver : inspectDriver) {
                if (!StringUtils.isEmpty(driver.getDriverFile())) {
                    // 如果上传了文件则对文件进行处理
                    String file = driver.getDriverFile();
                    if (file.contains("temp")) {
                        file = file.replace(uploadUrl, uploadFolder);
                        File oldFile = new File(file);
                        if (!oldFile.exists()) {
                            return "文件不存在";
                        }
                        //把文件移动到正式目录下
                        Calendar date = Calendar.getInstance();
                        String year = String.valueOf(date.get(Calendar.YEAR));
                        String month = String.valueOf(date.get(Calendar.MONTH) + 1);
                        String day = String.valueOf(date.get(Calendar.DATE));
                        String destFile = uploadFolder + "file/file/" + year + "/" + month + "/" + day + "/";
                        boolean flag = FileUtil.cutGeneralFile(file, destFile);
                        if (flag) {
                            // 文件名
                            String fileName = oldFile.getName();
                            destFile = destFile.replace(uploadFolder, "");
                            driver.setDriverFile(destFile + fileName);
                        } else {
                            return "文件保存失败了!";
                        }
                    }
                }
                Integer driverId = driver.getDriverId();
                TauvDriver driverInfo = iTauvDriverService.getInfoById(driverId);
                if (driverInfo != null) {
                    driver.setDriverName(driverInfo.getName());
                    driver.setDriverArea(driverInfo.getDriverArea());
                    BigDecimal bigDecimal = new BigDecimal(driverInfo.getLength());
                    driver.setDriverLength(bigDecimal);
                    driver.setDriverStart(driverInfo.getStartPoint());
                    driver.setDriverEnd(driverInfo.getEndPoint());
                } else {
                    //抛出异常
                    return "河流名称不存在";
                }
                driver.setId(0);
                driver.setInspectId(inspectId);
                // 任务来源：平台
                driver.setSource(1);
                driver.setInspectNo(inspect.getInspectNo());
                driver.setCreateTime(DateUtil.now());
                driver.setCreateUser(ShiroUtils.getAdminId());
                driver.setNum(i * 10);
                i++;
            }
        } else {
            return "请选择河流信息";
        }
        saveBatch(inspectDriver);
        return errorMsg;
    }

    /**
     * 巡检任务分页数据列表
     *
     * @param map
     * @param act
     * @return
     */
    @Override
    public Response getPageList(Map<String, Object> map, String act) {
        Integer pageNo = 1;
        if (!StringUtils.isEmpty(map)) {
            if (map.containsKey("page") && !"".equals(map.get("page")) && Integer.valueOf(map.get("page").toString()) > 0) {
                pageNo = Integer.valueOf(map.get("page").toString());
            }
            //每页数量
            if (map.containsKey("pageSize") && !"".equals(map.get("pageSize")) && Integer.valueOf(map.get("pageSize").toString()) > 0) {
                pageSize = Integer.valueOf(map.get("pageSize").toString());
            }
            //分配列表：期望巡检时间 任务列表：执行时间
            String inspectTime = "inspectTime";
            if (!StringUtils.isEmpty(map.get(inspectTime))) {
                String startTime = FunctionUtils.formatTime(Integer.valueOf(map.get(inspectTime).toString()), "yyyy-MM-dd") + " 00:00:00";
                if ("list".equals(act)) {
                    map.put("executionTime", startTime);
                    map.remove("inspectTime");
                } else {
                    map.put("inspectTime", startTime);
                    map.remove("executionTime");
                }
            }
            //区属
            map = FunctionUtils.dealArea(map);
        } else {
            map = new HashMap<>();
        }

        Page<TauvInspectDriverDTO> page = new Page<>(pageNo, pageSize);
        List<TauvInspectDriverDTO> data = inspectDriverMapper.getPageList(page, map);

        TauvInspect tauvInspect = new TauvInspect();
        if (data != null) {
            for (TauvInspectDriverDTO driver : data) {
                driver = getInfoDetail(driver);
            }
        }
        page.setRecords(data);
        return response.success(page);
    }

    /**
     * 根据编号获取数据
     *
     * @param id
     * @return
     */
    @Override
    public TauvInspectDriver getInfoById(Integer id) {
        return inspectDriverMapper.selectById(id);
    }

    /**
     * 获取待分配任务ID集合
     *
     * @param inspectDriverIds 待分配任务ID(多个逗号","分隔)
     * @return
     */
    @Override
    public Response getAssignTaskList(String inspectDriverIds) {
        // 查询条件
        QueryWrapper<TauvInspectDriver> queryWrapper = new QueryWrapper<>();
        // 待分配任务
        String arr[] = inspectDriverIds.split(",");
        ArrayList arrayList = new ArrayList();
        for (String s : arr) {
            arrayList.add(s);
        }
        queryWrapper.in("id", arrayList);
        queryWrapper.eq("mark", 1);

        // 查询数据
        List<TauvInspectDriver> inspectDriverList = inspectDriverMapper.selectList(queryWrapper);
        List<TauvInspectDriverListVo> inspectDriverListVoList = new ArrayList<>();
        List<String> inspectTypeList = new ArrayList<>();
        if (!inspectDriverList.isEmpty()) {
            inspectDriverList.forEach(item -> {
                TauvInspectDriverListVo inspectDriverListVo = new TauvInspectDriverListVo();
                // 拷贝属性
                BeanUtils.copyProperties(item, inspectDriverListVo);
                // 区划
                String cityName = iSysCityService.getCityName(item.getDriverArea());
                inspectDriverListVo.setDriverAreaName(cityName);
                // 图片地址
                inspectDriverListVo.setDriverFile(uploadUrl + inspectDriverListVo.getDriverFile());
                // 巡检时间
                TauvInspect inspectInfo = inspectMapper.selectById(item.getInspectId());
                if (inspectInfo != null) {
                    inspectDriverListVo.setInspectTime(inspectInfo.getInspectTime());
                    String inspectType = inspectInfo.getTypeList().get(inspectInfo.getType());
                    if (!inspectTypeList.contains(inspectType)) {
                        inspectTypeList.add(inspectType);
                    }
                }
                inspectDriverListVoList.add(inspectDriverListVo);
            });
        }

        // 返回结果
        Map<String, Object> result = new HashMap<>();
        result.put("inspectTypeList", inspectTypeList);
        result.put("inspectDriverList", inspectDriverListVoList);
        return response.success(result);
    }

    /**
     * 分配任务
     *
     * @param assignTaskDto
     * @return
     */
    @Override
    public Response assignTask(TauvInspectDriverAssignTaskDTO assignTaskDto) {
        // 验证河流是否为空
        if (assignTaskDto.getInspectDrivers() == null) {
            return response.failure("河流信息不存在");
        }
        // 执行时间
        if (assignTaskDto.getExecutionTime() == null) {
            return response.failure("执行时间不能为空");
        }
        // 无人机设备校验
        if (StringUtils.isEmpty(assignTaskDto.getEquipmentId())) {
            return response.failure("无人机设备不能为空");
        }
        // 飞手校验
        if (StringUtils.isEmpty(assignTaskDto.getInspectHand())) {
            return response.failure("飞手不能为空");
        }
        // 循环遍历河流信息
        Integer totalNum = 0;
        for (TauvInspectDriver inspectDriver : assignTaskDto.getInspectDrivers()) {
            if (inspectDriver.getId() == null || inspectDriver.getId() <= 0) {
                continue;
            }

            // 图片处理
            if (!StringUtils.isEmpty(inspectDriver.getDriverFile())) {
                // 如果上传了文件则对文件进行处理
                String file = inspectDriver.getDriverFile();
                if (file.contains("temp")) {
                    file = file.replace(uploadUrl, uploadFolder);
                    File oldFile = new File(file);
                    if (!oldFile.exists()) {
                        return response.failure("文件不存在");
                    }
                    //把文件移动到正式目录下
                    Calendar date = Calendar.getInstance();
                    String year = String.valueOf(date.get(Calendar.YEAR));
                    String month = String.valueOf(date.get(Calendar.MONTH) + 1);
                    String day = String.valueOf(date.get(Calendar.DATE));
                    String destFile = uploadFolder + "file/file/" + year + "/" + month + "/" + day + "/";
                    boolean flag = FileUtil.cutGeneralFile(file, destFile);
                    if (flag) {
                        // 文件名
                        String fileName = oldFile.getName();
                        destFile = destFile.replace(uploadFolder, "");
                        inspectDriver.setDriverFile(destFile + fileName);
                    } else {
                        return response.failure("文件保存失败了");
                    }
                }
            }

            // 执行时间
            inspectDriver.setExecutionTime(assignTaskDto.getExecutionTime());
            // 无人机设备
            inspectDriver.setEquipmentId(assignTaskDto.getEquipmentId());
            // 挂载设备
            if (!StringUtils.isEmpty(assignTaskDto.getMountequipmentId())) {
                inspectDriver.setMountequipmentId(assignTaskDto.getMountequipmentId());
            }
            // 飞手
            inspectDriver.setInspectHand(assignTaskDto.getInspectHand());
            // 设置已分配状态
            inspectDriver.setStatus(2);
            // 更新巡检河流信息
            boolean result = this.editData(inspectDriver);
            if (result) {
                totalNum++;
            }
        }
        if (totalNum == assignTaskDto.getInspectDrivers().size()) {
            return response.success("任务分配成功");
        }
        return response.failure("任务分配失败");
    }

    /**
     * 详情
     *
     * @param id
     * @return
     */
    @Override
    public Response detail(Integer id) {
        // 获取巡检任务河流
        TauvInspectDriver inspectDriver = inspectDriverMapper.selectById(id);
        if (inspectDriver == null) {
            return response.failure("巡检任务河流信息不存在");
        }
        // 拷贝属性
        TauvInspectDriverInfoVo inspectDriverInfoVo = new TauvInspectDriverInfoVo();
        BeanUtils.copyProperties(inspectDriver, inspectDriverInfoVo);

        // 获取区划描述
        String cityName = iSysCityService.getCityName(inspectDriver.getDriverArea());
        inspectDriverInfoVo.setDriverAreaName(cityName);
        // 获取航线文件
        if (!StringUtils.isEmpty(inspectDriver.getDriverFile())) {
            inspectDriverInfoVo.setDriverFile(uploadUrl + inspectDriver.getDriverFile());
        }
        // 飞手名称
        List<String> inspectHandName = iSysAdminService.getAdminNameByIds(inspectDriver.getInspectHand());
        inspectDriverInfoVo.setInspectHandName(org.apache.commons.lang3.StringUtils.join(inspectHandName.toArray(), "，"));

        // 获取巡检任务信息
        TauvInspect inspect = inspectMapper.selectById(inspectDriver.getInspectId());
        if (inspect == null) {
            return response.failure("巡检任务不存在");
        }
        inspectDriverInfoVo.setInspectTime(inspect.getInspectTime());

        // 获取无人机设备
        if (!StringUtils.isEmpty(inspectDriver.getEquipmentId())) {
            List<TauvEquipmentListVo> equipmentListVoList = equipmentService.getEquipmentListByIds(inspectDriver.getEquipmentId());
            inspectDriverInfoVo.setEquipmentList(equipmentListVoList);
        }

        // 获取挂载设备
        if (!StringUtils.isEmpty(inspectDriver.getMountequipmentId())) {
            List<TauvMountEquipmentListVo> mountEquipmentListVoList = mountEquipmentService.getMountEquipmentListByIds(inspectDriver.getMountequipmentId());
            inspectDriverInfoVo.setMountEquipmentList(mountEquipmentListVoList);
        }
        return response.success(inspectDriverInfoVo);
    }

    /**
     * 取消
     *
     * @param map
     * @return
     */
    @Override
    public Response cancel(Map<String, String> map) {
        if (StringUtils.isEmpty(map)) {
            return response.failure("参数不存在");
        }
        Integer id = Integer.valueOf(map.getOrDefault("id", ""));
        TauvInspectDriver info = getInfoById(id);
        if (StringUtils.isEmpty(info)) {
            return response.failure("数据不存在");
        }
        String status = info.getStatus().toString();
        String needStatus = "2";
        if (!(needStatus.equals(status))) {
            return response.failure("当前状态不可取消");
        }
        TauvInspectDriver inspectDriver = new TauvInspectDriver();
        inspectDriver.setId(id);
        inspectDriver.setStatus(Integer.valueOf("1"));
        inspectDriverMapper.updateById(inspectDriver);
        return response.success("操作成功");
    }

    /**
     * 执行任务
     *
     * @param task
     * @return
     */
    @Override
    public Response executeTask(ExecuteTask task) {
        Integer id = Integer.valueOf(task.getId());
        Integer adminId = ShiroUtils.getAdminId();
        TauvInspectDriver info = getInfoById(id);
        if (StringUtils.isEmpty(info)) {
            return response.failure("数据不存在");
        }
        Integer flightHand = info.getFlightHand();
        if (!flightHand.equals(adminId) && !flightHand.equals(1)) {
            response.failure("只有被分配的小组才有权限");
        }
        TauvInspectDriver inspectDriver = new TauvInspectDriver();
        inspectDriver.setWeather(StringUtils.isEmpty(task.getWeather()) ? "" : task.getWeather().trim());
        inspectDriver.setWind(StringUtils.isEmpty(task.getWind()) ? "" : task.getWind().trim());
        inspectDriver.setTemperature(StringUtils.isEmpty(task.getTemperature()) ? "" : task.getTemperature().trim());
        if (!StringUtils.isEmpty(task.getFlightHeight())) {
            inspectDriver.setFlightHeight(new BigDecimal(task.getFlightHeight()));
        }
        if (!StringUtils.isEmpty(task.getFlightSpeed())) {
            inspectDriver.setFlightSpeed(new BigDecimal(task.getFlightSpeed()));
        }
        if (!StringUtils.isEmpty(task.getFlightTime())) {
            inspectDriver.setFlightTime(new BigDecimal(task.getFlightTime()));
        }
        inspectDriver.setRemark(StringUtils.isEmpty(task.getRemark()) ? "" : task.getRemark().trim());
        inspectDriver.setStatus(3);
        inspectDriver.setFlightHand(adminId);
        inspectDriver.setExecutionTime(task.getExecutionTime());
        inspectDriver.setId(id);
        inspectDriverMapper.updateById(inspectDriver);
        return response.success("执行成功");
    }

    /**
     * 获取打印数据的基本信息
     *
     * @param id
     * @return
     */
    @Override
    public Response getPrintData(Integer id) {
        return this.detail(id);
    }

    /**
     * 获取数据详细数据
     *
     * @param driver
     * @return
     */
    private TauvInspectDriverDTO getInfoDetail(TauvInspectDriverDTO driver) {
        TauvInspect tauvInspect = new TauvInspect();
        Map<Integer, String> mapList = driver.getStatusList();
        Map<Integer, String> typeList = tauvInspect.getTypeList();

        if (StringUtils.isEmpty(driver.getType())) {
            driver.setTypeText("");
        } else {
            driver.setTypeText(typeList.get(driver.getType().intValue()));
        }

        driver.setStatusText(mapList.get(driver.getStatus().intValue()));

        SysAdmin adminInfo = iSysAdminService.getAdminInfo(driver.getCreateUser());
        driver.setFormatCreateUser(StringUtils.isEmpty(adminInfo) ? "" : adminInfo.getRealname());
        driver.setFormatCreateTime(DateUtil.format(driver.getCreateTime(), "yyyy-MM-dd HH:mm:ss"));
        driver.setFormatInspectTime(DateUtil.format(driver.getInspectTime(), "yyyy-MM-dd"));
        if (!StringUtils.isEmpty(driver.getExecutionTime())) {
            driver.setFormatExecutionTime(DateUtil.format(driver.getExecutionTime(), "yyyy-MM-dd"));
        }
        if (!StringUtils.isEmpty(driver.getDriverArea())) {
            SysCity cityInfo = iSysCityService.getInfoById(driver.getDriverArea());
            driver.setDriverAreaText(StringUtils.isEmpty(cityInfo) ? "" : cityInfo.getName());
        } else {
            driver.setDriverAreaText("");
        }
        //已完成时展示报告
        if (driver.getStatus() == 4) {
            wrapper = new QueryWrapper();
            wrapper.eq("mark", 1);
            wrapper.eq("inspect_driver_id", driver.getId());
            wrapper.last("limit 1");
            TauvReport report = iTauvReportService.getOne(wrapper);
            if (report != null) {
                driver.setReportId(report.getId());
            }
        }

        // 获取飞手信息
        List<String> inspectHandList = iSysAdminService.getAdminNameByIds(driver.getInspectHand());
        driver.setInspectHandName(Joiner.on("，").join(inspectHandList.toArray()));
        // 获取无人机设备名称、直播地址
        List<TauvEquipmentListVo> equipmentListVoList = equipmentService.getEquipmentListByIds(driver.getEquipmentId());
        if (equipmentListVoList.size() > 0) {
            List<String> equipmentNameList = new ArrayList<>();
            String liveUrl = "";
            for (TauvEquipmentListVo item : equipmentListVoList) {
                equipmentNameList.add(item.getName());
                if (!StringUtils.isEmpty(item.getLiveUrl())) {
                    liveUrl = item.getLiveUrl();
                }
            }
            driver.setEquipmentName(Joiner.on("，").join(equipmentNameList.toArray()));
            driver.setLiveUrl(liveUrl);
        }
        // 获取挂载设备名称
        List<String> mountEquipmentNameList = mountEquipmentService.getMountEquipmentNameByIds(driver.getMountequipmentId());
        driver.setMountequipmentName(Joiner.on("，").join(mountEquipmentNameList.toArray()));
        return driver;
    }

    /**
     * 判断任务是否已生成报告
     *
     * @param map
     * @return
     * @description
     */
    @Override
    public Response existReport(Map<String, Object> map) {
        if (StringUtils.isEmpty(map)) {
            return response.failure("参数不能为空");
        }
        String idStr = "id";
        if (map.containsKey(idStr) && !"".equals(map.get(idStr)) && map.get(idStr) != null) {
            Integer id = Integer.valueOf(map.get(idStr).toString());
            if (id <= 0) {
                return response.failure("任务id不合法");
            }
            QueryWrapper wrp = new QueryWrapper();
            wrp.eq("id", id);
            wrp.eq("mark", 1);
            TauvInspectDriver info = inspectDriverMapper.selectOne(wrp);
            if (info == null) {
                return response.failure("任务数据不存在");
            }
            //校验任务对应的报告是否已生成
            QueryWrapper wrapper = new QueryWrapper();
            wrapper.eq("mark", 1);
            wrapper.eq("num", info.getNum());
            wrapper.eq("inspect_no", info.getInspectNo());
            wrapper.eq("inspect_driver_id", id);
            TauvReport reportInfo = iTauvReportService.getOne(wrapper);
            if (reportInfo != null) {
                return response.success(true);
            } else {
                return response.success(false);
            }
        } else {
            return response.failure("任务id不存在");
        }
    }

    /**
     * 删除任务
     *
     * @param map
     * @return
     * @description 校验任务是否已存在报告，如已存在，则不可删除
     */
    @Override
    public Response dropTask(Map<String, Object> map) {
        if (StringUtils.isEmpty(map)) {
            return response.failure("参数不能为空");
        }
        String idStr = "id";
        if (map.containsKey(idStr) && !"".equals(map.get(idStr)) && map.get(idStr) != null) {
            Integer id = Integer.valueOf(map.get(idStr).toString());
            if (id <= 0) {
                return response.failure("任务id不合法");
            }
            QueryWrapper wrp = new QueryWrapper();
            wrp.eq("id", id);
            wrp.eq("mark", 1);
            TauvInspectDriver info = inspectDriverMapper.selectOne(wrp);
            if (info == null) {
                return response.failure("任务数据不存在");
            }
            //校验任务对应的报告是否已生成
            QueryWrapper wrapper = new QueryWrapper();
            wrapper.eq("mark", 1);
            wrapper.eq("num", info.getNum());
            wrapper.eq("inspect_no", info.getInspectNo());
            wrapper.eq("inspect_driver_id", id);
            TauvReport reportInfo = iTauvReportService.getOne(wrapper);
            if (reportInfo != null) {
                return response.failure("该任务已经生成报告，请先删除报告后再处理。");
            }
            info.setMark(0);
            inspectDriverMapper.updateById(info);
            return response.success("操作成功");
        } else {
            return response.failure("任务id不存在");
        }
    }

    /**
     * 重处理任务
     *
     * @param map
     * @return
     */
    @Override
    public Response reProcess(Map<String, Object> map) {
        if (StringUtils.isEmpty(map)) {
            return response.failure("参数不能为空");
        }
        String idStr = "id";
        if (map.containsKey(idStr) && !"".equals(map.get(idStr)) && map.get(idStr) != null) {
            Integer id = Integer.valueOf(map.get(idStr).toString());
            if (id <= 0) {
                return response.failure("任务id不合法");
            }
            QueryWrapper wrp = new QueryWrapper();
            wrp.eq("id", id);
            wrp.eq("mark", 1);
            TauvInspectDriver info = inspectDriverMapper.selectOne(wrp);
            if (info == null) {
                return response.failure("任务数据不存在");
            }
            info.setStatus(Integer.valueOf("3"));
            inspectDriverMapper.updateById(info);
            return response.success("操作成功");
        } else {
            return response.failure("任务id不存在");
        }
    }
}
