package com.taauav.admin.service.impl;


import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.taauav.admin.dto.TauvInspectQuestionDTO;
import com.taauav.admin.entity.SysCity;
import com.taauav.admin.mapper.SysCityMapper;
import com.taauav.admin.vo.TauvInspectQuestionVo;
import com.taauav.common.bean.Response;
import com.taauav.common.core.mps.BaseQuery;
import com.taauav.common.service.impl.BaseServiceImpl;
import com.taauav.common.util.FunctionUtils;
import com.taauav.common.util.StringUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.stereotype.Service;
import com.taauav.admin.mapper.TauvInspectQuestionMapper;
import com.taauav.admin.entity.TauvInspectQuestion;
import com.taauav.admin.service.ITauvInspectQuestionService;

import javax.annotation.Resource;
import java.math.BigInteger;

/**
 * 问题管理Service业务层处理
 *
 * @author daixiantong
 * @date 20200518
 */
@Service
public class TauvInspectQuestionServiceImpl extends BaseServiceImpl<TauvInspectQuestionMapper, TauvInspectQuestion> implements ITauvInspectQuestionService
{
    @Resource
    private Response response;
    @Autowired
    private SysCityMapper cityMapper;
    @Value("${server.IMAGE_URL}")
    private String imageUrl;

    /**
     * 自定义分页列表
     * @param inspectQuestionDTO
     * @param query
     * @return
     */
    @Override
    public Response selectQuestionPage(TauvInspectQuestionDTO inspectQuestionDTO, BaseQuery query) {
        IPage page = new Page(query.getPage(), query.getPageSize());
        IPage<TauvInspectQuestionVo> list = baseMapper.selectQuestionList(page, inspectQuestionDTO);
        if (list.getRecords().size() > 0) {
            for (TauvInspectQuestionVo item : list.getRecords()) {
                // 状态描述
                item.setStatusText(FunctionUtils.getArrayText(item.getStatus().toString(), item.statusList()));
                // 区属名称
                if (item.getDriverArea().compareTo(BigInteger.ZERO) > 0) {
                    SysCity city = cityMapper.selectById(item.getDriverArea());
                    item.setAreaName(city != null ? city.getName() : "");
                }
                // 图片路径处理
                if (StringUtils.isNotEmpty(item.getImgUrl()) && !item.getImgUrl().contains("http")) {
                    item.setImgUrl(imageUrl + item.getImgUrl());
                }
            }
        }
        return response.success(list);
    }

    @Override
    public Response getQuestionData(Integer id) {
        if (id == null || id <= 0) {
            return response.failure("问题id不能为空");
        }
        TauvInspectQuestionVo detail = baseMapper.getQuestionDetailInfo(id);
        if (detail != null) {
            detail.setStatusText(FunctionUtils.getArrayText(detail.getStatus().toString(), detail.statusList()));
            if (StringUtils.isNotEmpty(detail.getImgUrl()) && !detail.getImgUrl().contains("http")) {
                detail.setImgUrl(imageUrl + detail.getImgUrl());
            }
            if (StringUtils.isNotEmpty(detail.getHandlerImage()) && !detail.getHandlerImage().contains("http")) {
                detail.setHandlerImage(imageUrl + detail.getHandlerImage());
            }
        }
        return response.success(detail);
    }
}
