package com.taauav.admin.service.impl;

import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.taauav.admin.entity.TauvWaterAlarm;
import com.taauav.admin.entity.TauvWaterAlarmInfo;
import com.taauav.admin.mapper.TauvWaterAlarmInfoMapper;
import com.taauav.admin.mapper.TauvWaterAlarmMapper;
import com.taauav.admin.query.TauvWaterAlarmQuery;
import com.taauav.admin.service.ITauvWaterAlarmService;
import com.taauav.admin.vo.TauvWaterAlarmVo;
import com.taauav.common.bean.Response;
import com.taauav.common.service.impl.BaseServiceImpl;
import com.taauav.common.util.FunctionUtils;
import com.taauav.common.util.StringUtils;
import org.springframework.beans.BeanUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import javax.annotation.Resource;
import java.math.BigInteger;
import java.util.List;

/**
 * 水质超标报警服务类
 *
 * @author daixiantong
 * @date 2020-05-14
 */
@Service
public class TauvWaterAlarmServiceImpl extends BaseServiceImpl<TauvWaterAlarmMapper, TauvWaterAlarm> implements ITauvWaterAlarmService {
    @Resource
    private Response response;
    @Autowired
    private TauvWaterAlarmInfoMapper waterAlarmInfoMapper;

    /**
     * 自定义分页
     * @param waterAlarmQuery
     * @return
     */
    @Override
    public Response selectPageList(TauvWaterAlarmQuery waterAlarmQuery) {
        IPage<TauvWaterAlarm> page = new Page<>(waterAlarmQuery.getPage(), waterAlarmQuery.getPageSize());
        QueryWrapper wrapper = new QueryWrapper();
        wrapper.eq("mark", 1);
        if (StringUtils.isNotEmpty(waterAlarmQuery.getAlarmSn())) {
            wrapper.like("alarm_sn", waterAlarmQuery.getAlarmSn());
        }
        if (StringUtils.isNotEmpty(waterAlarmQuery.getDriverName())) {
            wrapper.like("driver_name", waterAlarmQuery.getDriverName());
        }
        if (StringUtils.isNotNull(waterAlarmQuery.getDriverArea()) && waterAlarmQuery.getDriverArea().compareTo(BigInteger.ZERO) > 0) {
            wrapper.eq("driver_area", waterAlarmQuery.getDriverArea());
        }
        if (StringUtils.isNotNull(waterAlarmQuery.getStatus()) && waterAlarmQuery.getStatus() > 0) {
            wrapper.eq("status", waterAlarmQuery.getStatus());
        }
        if (waterAlarmQuery.getStartTime() != null && waterAlarmQuery.getEndTime() != null) {
            String beginTime = waterAlarmQuery.getStartTime().toString();
            String endTime = waterAlarmQuery.getEndTime().toString();
            if (StringUtils.isNotEmpty(beginTime) && StringUtils.isNotEmpty(endTime)) {
                wrapper.between("alarm_time", beginTime + " 00:00:00", endTime + " 23:59:59");
            }
        }
        IPage<TauvWaterAlarm> list = baseMapper.selectPage(page, wrapper);
        if (list.getRecords().size() > 0) {
            for (TauvWaterAlarm data : list.getRecords()) {
                data.setStatusText(FunctionUtils.getArrayText(data.getStatus().toString(), data.statusList()));
            }
        }
        return response.success(list);
    }

    /**
     * 获取详情数据
     * @param id
     * @return
     */
    @Override
    public TauvWaterAlarmVo getDetail(Integer id) {
        TauvWaterAlarm waterAlarm = getInfo(id);
        if (waterAlarm != null) {
            waterAlarm.setStatusText(FunctionUtils.getArrayText(waterAlarm.getStatus().toString(), waterAlarm.statusList()));
        }
        QueryWrapper wrapper = new QueryWrapper();
        wrapper.eq("mark", 1);
        wrapper.eq("water_alarm_id", id);
        wrapper.orderByAsc("id");
        List<TauvWaterAlarmInfo> infoList = waterAlarmInfoMapper.selectList(wrapper);
        TauvWaterAlarmVo waterAlarmVo = new TauvWaterAlarmVo();
        BeanUtils.copyProperties(waterAlarm, waterAlarmVo, TauvWaterAlarmVo.class);
        waterAlarmVo.setInfoList(infoList);
        return waterAlarmVo;
    }
}
