package com.taauav.front.service.impl;

import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.taauav.common.bean.Response;
import com.taauav.front.entity.UserAuthGroupAccess;
import com.taauav.front.mapper.UserAuthGroupAccessMapper;
import com.taauav.front.service.IUserAuthGroupAccessService;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.util.StringUtils;

import java.util.*;

/**
 * <p>
 * 用户组明细表 服务实现类
 * </p>
 *
 * @author 鲲鹏
 * @since 2020-05-16
 */
@Service
public class UserAuthGroupAccessServiceImpl extends ServiceImpl<UserAuthGroupAccessMapper, UserAuthGroupAccess> implements IUserAuthGroupAccessService {

    @Autowired
    private Response response;
    @Autowired
    private UserAuthGroupAccessMapper userAuthGroupAccessMapper;

    @Override
    public List<UserAuthGroupAccess> getListByGroupid(Integer groupId) {
        QueryWrapper<UserAuthGroupAccess> queryWrapper = new QueryWrapper<>();
        queryWrapper.eq("group_id", groupId);
        return userAuthGroupAccessMapper.selectList(queryWrapper);
    }

    @Override
    public Map<String, String> add(Integer adminId, String groupIds) {
        Map<String, String> result = new HashMap<>();
        result.put("code", "500");
        if (StringUtils.isEmpty(adminId)) {
            result.put("msg", "请选择用户");
            return result;
        }
        if (StringUtils.isEmpty(groupIds)) {
            result.put("msg", "请选择角色");
            return result;
        }
        String[] groupIdList = groupIds.split(",");
        Set<String> requestGroupIds = new HashSet<String>(Arrays.asList(groupIdList));

        Map<String, String> hasGroupMap = new HashMap<>();
        List<UserAuthGroupAccess> groupAccessList = getListByAdmin(adminId);
        if (groupAccessList != null) {
            for (UserAuthGroupAccess userAuthGroupAccess : groupAccessList) {
                String val = userAuthGroupAccess.getGroupId().toString();
                hasGroupMap.put(val, val);
            }
        }
        //判断权限是否已存在，不存在时添加
        for (String groupId : groupIdList) {
            if (hasGroupMap.containsKey(groupId)) {
                continue;
            } else {
                UserAuthGroupAccess insertData = new UserAuthGroupAccess();
                insertData.setUid(adminId);
                insertData.setGroupId(Integer.valueOf(groupId));
                userAuthGroupAccessMapper.insert(insertData);
            }
        }
        //删除已存在多余角色权限
        for (String hGroup : hasGroupMap.values()) {
            if (!requestGroupIds.contains(hGroup)) {
                QueryWrapper<UserAuthGroupAccess> delSet = new QueryWrapper<>();
                delSet.eq("uid", adminId);
                delSet.eq("group_id", Integer.valueOf(hGroup));
                userAuthGroupAccessMapper.delete(delSet);
            }
        }
        result.put("code", "200");
        result.put("msg", "ok");
        return result;
    }

    @Override
    public List<UserAuthGroupAccess> getListByAdmin(Integer adminId) {
        QueryWrapper<UserAuthGroupAccess> queryWrapper = new QueryWrapper<>();
        queryWrapper.eq("uid", adminId);
        return userAuthGroupAccessMapper.selectList(queryWrapper);
    }

}
