package com.taauav.front.service.impl;

import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.taauav.admin.dto.TauvInspectDriverDTO;
import com.taauav.admin.entity.*;
import com.taauav.admin.mapper.TauvInspectDriverMapper;
import com.taauav.admin.mapper.TauvInspectMapper;
import com.taauav.admin.service.*;
import com.taauav.admin.vo.TauvEquipmentListVo;
import com.taauav.admin.vo.TauvMountEquipmentListVo;
import com.taauav.admin.vo.inspectdriver.TauvInspectDriverInfoVo;
import com.taauav.common.bean.Response;
import com.taauav.common.service.impl.BaseServiceImpl;
import com.taauav.common.util.DateUtil;
import com.taauav.common.util.FileUtil;
import com.taauav.common.util.FunctionUtils;
import com.taauav.common.util.ShiroUtils;
import com.taauav.front.mapper.UserAdminMapper;
import com.taauav.front.service.IUserAdminService;
import com.taauav.front.service.IUserInspectDriverService;
import com.taauav.front.vo.UserAdminVo;
import org.springframework.beans.BeanUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.stereotype.Service;
import org.springframework.util.StringUtils;

import java.io.File;
import java.math.BigDecimal;
import java.util.*;

/**
 * <p>
 * 外包人员表 服务实现类
 * </p>
 *
 * @author 鲲鹏
 * @since 2020-05-14
 */
@Service
public class UserInspectDriverServiceImpl extends BaseServiceImpl<TauvInspectDriverMapper, TauvInspectDriver> implements IUserInspectDriverService {

    @Autowired
    private UserAdminMapper userAdminMapper;

    @Autowired
    private IUserAdminService userAdminService;

    @Autowired
    private TauvInspectDriverMapper inspectDriverMapper;

    @Value("${spring.pageSize}")
    private Integer pageSize;
    @Value("${server.UPLOAD_URL}")
    private String uploadUrl;
    @Value("${file.uploadFolder}")
    private String uploadFolder;

    @Autowired
    private Response response;

    @Autowired
    private ISysCityService iSysCityService;

    @Autowired
    private ISysAdminService iSysAdminService;

    @Autowired
    private ITauvReportService iTauvReportService;

    @Autowired
    private ITauvEquipmentService equipmentService;

    @Autowired
    private ITauvMountEquipmentService mountEquipmentService;

    @Autowired
    private ITauvDriverService iTauvDriverService;

    @Autowired
    private TauvInspectMapper inspectMapper;

    @Autowired
    private ITauvInspectFileService inspectFileService;

    @Override
    public String batchAddData(TauvInspect inspect) {
        List<TauvInspectDriver> inspectDriver = inspect.getInspectDriverList();
        Integer inspectId = inspect.getId();
        String errorMsg = "";
        if (inspectDriver != null && !"".equals(inspectDriver) && inspectDriver.size() > 0) {
            int i = 1;
            for (TauvInspectDriver driver : inspectDriver) {
                if (!StringUtils.isEmpty(driver.getDriverFile())) {
                    // 如果上传了文件则对文件进行处理
                    String file = driver.getDriverFile();
                    if (file.contains("temp")) {
                        file = file.replace(uploadUrl, uploadFolder);
                        File oldFile = new File(file);
                        if (!oldFile.exists()) {
                            return "文件不存在";
                        }
                        //把文件移动到正式目录下
                        Calendar date = Calendar.getInstance();
                        String year = String.valueOf(date.get(Calendar.YEAR));
                        String month = String.valueOf(date.get(Calendar.MONTH) + 1);
                        String day = String.valueOf(date.get(Calendar.DATE));
                        String destFile = uploadFolder + "file/file/" + year + "/" + month + "/" + day + "/";
                        boolean flag = FileUtil.cutGeneralFile(file, destFile);
                        if (flag) {
                            // 文件名
                            String fileName = oldFile.getName();
                            destFile = destFile.replace(uploadFolder, "");
                            driver.setDriverFile(destFile + fileName);
                        } else {
                            return "文件保存失败了!";
                        }
                    }
                }
                Integer driverId = driver.getDriverId();
                TauvDriver driverInfo = iTauvDriverService.getInfoById(driverId);
                if (driverInfo != null) {
                    driver.setDriverName(driverInfo.getName());
                    driver.setDriverArea(driverInfo.getDriverArea());
                    BigDecimal bigDecimal = new BigDecimal(driverInfo.getLength());
                    driver.setDriverLength(bigDecimal);
                    driver.setDriverStart(driverInfo.getStartPoint());
                    driver.setDriverEnd(driverInfo.getEndPoint());
                } else {
                    //抛出异常
                    return "河流名称不存在";
                }
                driver.setId(0);
                driver.setInspectId(inspectId);
                // 任务来源：客户
                driver.setSource(2);
                driver.setInspectNo(inspect.getInspectNo());
                driver.setCreateTime(DateUtil.now());
                driver.setCreateUser(ShiroUtils.getAdminId());
                driver.setNum(i * 10);
                i++;
            }
        } else {
            return "请选择河流信息";
        }
        saveBatch(inspectDriver);
        return errorMsg;
    }

    /**
     * 获取巡检任务列表
     *
     * @param map 查询条件
     * @return
     */
    @Override
    public Response getList(Map<String, Object> map) {
        Integer pageNo = 1;
        if (!org.springframework.util.StringUtils.isEmpty(map)) {
            if (map.containsKey("page") && !"".equals(map.get("page")) && Integer.valueOf(map.get("page").toString()) > 0) {
                pageNo = Integer.valueOf(map.get("page").toString());
            }
            //每页数量
            if (map.containsKey("pageSize") && !"".equals(map.get("pageSize")) && Integer.valueOf(map.get("pageSize").toString()) > 0) {
                pageSize = Integer.valueOf(map.get("pageSize").toString());
            }
            //分配列表：期望巡检时间 任务列表：执行时间
            String inspectTime = "inspectTime";
            if (!StringUtils.isEmpty(map.get(inspectTime))) {
                String startTime = FunctionUtils.formatTime(Integer.valueOf(map.get(inspectTime).toString()), "yyyy-MM-dd") + " 00:00:00";
                map.put("executionTime", startTime);
            }
            //区属
            map = FunctionUtils.dealArea(map);
        } else {
            map = new HashMap<>();
        }
        Page<TauvInspectDriverDTO> page = new Page<>(pageNo, pageSize);
        List<TauvInspectDriverDTO> data = inspectDriverMapper.getPageList(page, map);
        TauvInspect tauvInspect = new TauvInspect();
        if (data != null) {
            for (TauvInspectDriverDTO driver : data) {
                driver = getInfoDetail(driver);
            }
        }
        page.setRecords(data);
        return response.success(page);
    }

    /**
     * 获取数据详细数据
     *
     * @param driver
     * @return
     */
    private TauvInspectDriverDTO getInfoDetail(TauvInspectDriverDTO driver) {
        TauvInspect tauvInspect = new TauvInspect();
        Map<Integer, String> mapList = driver.getStatusList();
        Map<Integer, String> typeList = tauvInspect.getTypeList();
        if (StringUtils.isEmpty(driver.getType())) {
            driver.setTypeText("");
        } else {
            driver.setTypeText(typeList.get(driver.getType().intValue()));
        }
        driver.setStatusText(mapList.get(driver.getStatus().intValue()));
        if (driver.getSource() == 1) {
            // 平台发起的任务
            SysAdmin adminInfo = iSysAdminService.getAdminInfo(driver.getCreateUser());
            driver.setFormatCreateUser(StringUtils.isEmpty(adminInfo) ? "" : adminInfo.getRealname());
        } else {
            // 客户发起的任务
            UserAdminVo adminInfo = userAdminService.getAdminInfo(driver.getCreateUser());
            driver.setFormatCreateUser(StringUtils.isEmpty(adminInfo) ? "" : adminInfo.getRealname());
        }
        driver.setFormatCreateTime(DateUtil.format(driver.getCreateTime(), "yyyy-MM-dd HH:mm:ss"));
        driver.setFormatInspectTime(DateUtil.format(driver.getInspectTime(), "yyyy-MM-dd"));
        if (!StringUtils.isEmpty(driver.getExecutionTime())) {
            driver.setFormatExecutionTime(DateUtil.format(driver.getExecutionTime(), "yyyy-MM-dd"));
        }
        if (!StringUtils.isEmpty(driver.getDriverArea())) {
            SysCity cityInfo = iSysCityService.getInfoById(driver.getDriverArea());
            driver.setDriverAreaText(StringUtils.isEmpty(cityInfo) ? "" : cityInfo.getName());
        } else {
            driver.setDriverAreaText("");
        }
        //已完成时展示报告
        if (driver.getStatus() == 4) {
            QueryWrapper wrapper = new QueryWrapper();
            wrapper.eq("mark", 1);
            wrapper.eq("inspect_driver_id", driver.getId());
            wrapper.last("limit 1");
            TauvReport report = iTauvReportService.getOne(wrapper);
            if (report != null) {
                driver.setReportId(report.getId());
            }
        }

        // 获取飞手信息
        List<String> inspectHandList = iSysAdminService.getAdminNameByIds(driver.getInspectHand());
        driver.setInspectHandName(org.apache.commons.lang3.StringUtils.join(inspectHandList.toArray(), "，"));
        // 获取无人机设备名称
        List<String> equipmentNameList = equipmentService.getEquipmentNameByIds(driver.getEquipmentId());
        driver.setEquipmentName(org.apache.commons.lang3.StringUtils.join(equipmentNameList.toArray(), "，"));
        // 获取挂载设备名称
        List<String> mountEquipmentNameList = mountEquipmentService.getMountEquipmentNameByIds(driver.getMountequipmentId());
        driver.setMountequipmentName(org.apache.commons.lang3.StringUtils.join(mountEquipmentNameList.toArray(), "，"));
        return driver;
    }

    /**
     * 获取任务详情
     *
     * @param id 任务ID
     * @return
     */
    @Override
    public Response detail(Integer id) {
        // 获取巡检任务河流
        TauvInspectDriver inspectDriver = inspectDriverMapper.selectById(id);
        if (inspectDriver == null) {
            return response.failure("巡检任务河流信息不存在");
        }
        // 拷贝属性
        TauvInspectDriverInfoVo inspectDriverInfoVo = new TauvInspectDriverInfoVo();
        BeanUtils.copyProperties(inspectDriver, inspectDriverInfoVo);

        // 获取区划描述
        String cityName = iSysCityService.getCityName(inspectDriver.getDriverArea());
        inspectDriverInfoVo.setDriverAreaName(cityName);
        // 获取航线文件
        if (!StringUtils.isEmpty(inspectDriver.getDriverFile())) {
            inspectDriverInfoVo.setDriverFile(uploadUrl + inspectDriver.getDriverFile());
        }
        // 飞手名称
        List<String> inspectHandName = iSysAdminService.getAdminNameByIds(inspectDriver.getInspectHand());
        inspectDriverInfoVo.setInspectHandName(org.apache.commons.lang3.StringUtils.join(inspectHandName.toArray(), "，"));

        // 获取巡检任务信息
        TauvInspect inspect = inspectMapper.selectById(inspectDriver.getInspectId());
        if (inspect == null) {
            return response.failure("巡检任务不存在");
        }
        inspectDriverInfoVo.setInspectTime(inspect.getInspectTime());

        // 获取无人机设备
        if (!StringUtils.isEmpty(inspectDriver.getEquipmentId())) {
            List<TauvEquipmentListVo> equipmentListVoList = equipmentService.getEquipmentListByIds(inspectDriver.getEquipmentId());
            inspectDriverInfoVo.setEquipmentList(equipmentListVoList);
        }

        // 获取挂载设备
        if (!StringUtils.isEmpty(inspectDriver.getMountequipmentId())) {
            List<TauvMountEquipmentListVo> mountEquipmentListVoList = mountEquipmentService.getMountEquipmentListByIds(inspectDriver.getMountequipmentId());
            inspectDriverInfoVo.setMountEquipmentList(mountEquipmentListVoList);
        }
        // 获取巡检任务航拍图片列表
        List<TauvInspectFile> inspectFileList = inspectFileService.getInspectFileListByInspectDriverId(inspectDriver.getId());
        inspectDriverInfoVo.setInspectFileList(inspectFileList);
        return response.success(inspectDriverInfoVo);
    }
}
