package com.taauav.tool.service;

import com.taauav.common.config.GenConfig;
import com.taauav.common.core.text.Convert;
import com.taauav.common.service.impl.BaseServiceImpl;
import com.taauav.common.util.ShiroUtils;
import com.taauav.common.util.StringUtils;
import com.taauav.tool.entity.GenTable;
import com.taauav.tool.entity.GenTableColumn;
import com.taauav.tool.mapper.GenTableMapper;
import com.taauav.tool.util.GenUtils;
import com.taauav.tool.util.VelocityInitializer;
import com.taauav.tool.util.VelocityUtils;
import org.apache.velocity.Template;
import org.apache.velocity.VelocityContext;
import org.apache.velocity.app.Velocity;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.io.File;
import java.io.FileNotFoundException;
import java.io.PrintWriter;
import java.util.List;

/**
 * 业务 服务层实现
 *
 * @author dyg
 */
@Service
public class GenTableServiceImpl extends BaseServiceImpl<GenTableMapper, GenTable> implements IGenTableService {

    @Autowired
    private IGenTableColumnService genTableColumnService;

    @Override
    public void generatorCode(String tableName) {
        GenTable table = getBaseMapper().selectGenTableByName(tableName);
        //查询列信息
        List<GenTableColumn> colums = table.getColumns();
        setPkColumn(table, colums);

        VelocityInitializer.initVelocity();
        VelocityContext context = VelocityUtils.prepareContext(table);
        //获取模板列表
        List<String> templates = VelocityUtils.getTemplateList(table.getTplCategory());
        String rootPath = System.getProperty("user.dir");
        for (String template : templates) {
            String fileName = VelocityUtils.getFileName(template, table);
            if (StringUtils.isEmpty(fileName)) {
                continue;
            }
            //渲染模板
            Template tpl = Velocity.getTemplate(template, GenConfig.UTF8);
            PrintWriter write = null;
            try {
                File f = new File(fileName);
                if (!f.getParentFile().isDirectory()) {
                    f.getParentFile().mkdirs();
                }
                write = new PrintWriter(fileName);
                tpl.merge(context, write);
                write.flush();
            } catch (FileNotFoundException e) {
                e.printStackTrace();
            } finally {
                write.close();
            }
        }
    }

    @Override
    public void importGenTable(String tables) {
        String[] tableNames = Convert.toStrArray(tables);
        // 查询表信息
        List<GenTable> tableList = selectDbTableListByNames(tableNames);
        Integer operName = ShiroUtils.getAdminId();
        for (GenTable table : tableList)
        {
            try
            {
                String tableName = table.getTableName();
                GenUtils.initTable(table, operName);
                boolean row = save(table);
                if (row)
                {
                    // 保存列信息
                    List<GenTableColumn> genTableColumns = genTableColumnService.selectDbTableColumnsByName(tableName);
                    for (GenTableColumn column : genTableColumns)
                    {
                        GenUtils.initColumnField(column, table);
                        genTableColumnService.insertGenTableColumn(column);
                    }
                }
            }
            catch (Exception e)
            {
                log.error("表名 " + table.getTableName() + " 导入失败：", e);
            }
        }
    }

    public List<GenTable> selectDbTableListByNames(String[] tableNames) {
        List<GenTable> tableList = getBaseMapper().selectDbTableListByNames(tableNames);
        return tableList;
    }

    /**
     * 设置主键列信息
     * @param genTable
     * @param columns
     */
    public void setPkColumn(GenTable genTable, List<GenTableColumn> columns) {
        for (GenTableColumn column : columns) {
            if (column.isPk()) {
                genTable.setPkColumn(column);
                break;
            }
        }
        if (StringUtils.isNull(genTable.getPkColumn())) {
            genTable.setPkColumn(columns.get(0));
        }
    }

}
