package com.taauav.tool.util;

import com.alibaba.fastjson.JSONObject;
import com.taauav.common.constant.GenConstants;
import com.taauav.common.util.DateUtil;
import com.taauav.common.util.StringUtils;
import com.taauav.tool.entity.GenTable;
import com.taauav.tool.entity.GenTableColumn;
import org.apache.velocity.VelocityContext;

import java.util.ArrayList;
import java.util.HashSet;
import java.util.List;

public class VelocityUtils
{
    /** 项目空间路径 */
    private static final String PROJECT_PATH = System.getProperty("user.dir")+"/src/main/java";

    public static VelocityContext prepareContext(GenTable genTable)
    {
        String moduleName = genTable.getModuleName();
        String businessName = genTable.getBusinessName();
        String packageName = genTable.getPackageName();
        String tplCategory = genTable.getTplCategory();
        String funcationName = genTable.getFunctionName();

        List<GenTableColumn> columnList = genTable.getColumns();
        List<GenTableColumn> needColumnList = new ArrayList<>();
        for (int i=0; i < columnList.size(); i++) {
            GenTableColumn column = columnList.get(i);
            String columnName = column.getColumnName();

            if (columnName.equals("status") || columnName.equals("create_user") ||
                    columnName.equals("create_time") || columnName.equals("update_user") ||
                    columnName.equals("update_time") || columnName.equals("mark")) {
                continue;
            } else {
                needColumnList.add(column);
            }
        }

        VelocityContext velocityContext = new VelocityContext();
        velocityContext.put("tplCategory", tplCategory);
        velocityContext.put("tableName", genTable.getTableName());
        velocityContext.put("functionName", funcationName);
        velocityContext.put("className", StringUtils.uncapitalize(genTable.getClassName()));
        velocityContext.put("ClassName", genTable.getClassName());
        velocityContext.put("moduleName", moduleName);
        velocityContext.put("businessName", StringUtils.uncapitalize(businessName));
        velocityContext.put("packageName", packageName);
        velocityContext.put("author", genTable.getFunctionAuthor());
        velocityContext.put("datetime", DateUtil.today());
        velocityContext.put("pkColumn", genTable.getPkColumn());
        velocityContext.put("importList", getImportList(genTable.getColumns()));
        velocityContext.put("permissionPrefix", getPermissionPrefix(moduleName,businessName));
        velocityContext.put("columns", needColumnList);
        velocityContext.put("table", genTable);
        return velocityContext;
    }


    /**
     * 获取模板信息
     * @param tplCategory
     * @return
     */
    public static List<String> getTemplateList(String tplCategory)
    {
        List<String> templates = new ArrayList<>();
        templates.add("vm/java/entity.java.vm");
        templates.add("vm/java/mapper.java.vm");
        templates.add("vm/java/wrapper.java.vm");
        templates.add("vm/java/service.java.vm");
        templates.add("vm/java/serviceImpl.java.vm");
        templates.add("vm/java/controller.java.vm");
        templates.add("vm/xml/mapper.xml.vm");
        templates.add("vm/sql/sql.vm");
        return templates;
    }
    /**
     * 获取文件名称
     * @param template
     * @param genTable
     * @return
     */
    public static String getFileName(String template, GenTable genTable)
    {
        //文件名称
        String fileName = "";

        String packageName =StringUtils.replace( genTable.getPackageName(), ".", "/");


        String className = genTable.getClassName();

        String bussinessName = genTable.getBusinessName();

        String javaPath = PROJECT_PATH;

        if (template.contains("entity.java.vm"))
        {
            fileName = StringUtils.format("{}/{}/entity/{}.java", javaPath,packageName, className);
        } else if (template.contains("mapper.java.vm"))
        {
            fileName = StringUtils.format("{}/{}/mapper/{}Mapper.java", javaPath,packageName, className);
        } else if (template.contains("service.java.vm"))
        {
            fileName = StringUtils.format("{}/{}/service/I{}Service.java", javaPath,packageName, className);
        } else if (template.contains("serviceImpl.java.vm"))
        {
            fileName = StringUtils.format("{}/{}/service/impl/{}ServiceImpl.java", javaPath,packageName, className);
        } else if (template.contains("controller.java.vm"))
        {
            fileName = StringUtils.format("{}/{}/controller/{}Controller.java", javaPath,packageName, className);
        } else if (template.contains("mapper.xml.vm"))
        {
            fileName = StringUtils.format("{}/{}/mapper/{}Mapper.xml", javaPath,packageName, className);
        } else if (template.contains("sql.vm"))
        {
            fileName = PROJECT_PATH + "/" + bussinessName + "/Menu.sql";
        }
        return fileName;
    }

    /**
     * 获取包前缀
     * @param packageName
     * @return
     */
    public static String getPackagePrefix(String packageName)
    {
        int lastIndex = packageName.lastIndexOf(".");
        String basePackage = StringUtils.substring(packageName,0, lastIndex);
        return basePackage;
    }

    /**
     * 根据列类型获取导入包
     * @param columns 列集合
     * @return
     */
    public static HashSet<String> getImportList(List<GenTableColumn> columns)
    {
        HashSet<String> importList = new HashSet<>();
        for (GenTableColumn column : columns)
        {
            if (!column.isSuperColumn() && GenConstants.TYPE_DATE.equals(column.getJavaType()))
            {
                importList.add("java.util.Date");
            } else if(!column.isSuperColumn() && GenConstants.TYPE_BIGDECIMAL.equals(column.getJavaType()))
            {
                importList.add("java.math.BigDecimal");
            }else if(!column.isSuperColumn() && GenConstants.HTML_DATETIME.equals(column.getJavaType()))
            {
                importList.add("java.util.Date");
            }
        }
        return importList;
    }

    /**
     * 获取权限前缀
     * @param moduleName
     * @param businessName
     * @return
     */
    public static String getPermissionPrefix(String moduleName, String businessName)
    {
        return StringUtils.format("{}:{}", moduleName, businessName);
    }

    /**
     * 根据需要在哪一列上面显示展开按钮
     * @param genTable
     * @return
     */
    public static int getExpandColumn(GenTable genTable)
    {
        String options = genTable.getOptions();
        JSONObject paramsObj = JSONObject.parseObject(options);
        String treeName = paramsObj.getString(GenConstants.TREE_NAME);
        int num = 0;
        for (GenTableColumn column : genTable.getColumns())
        {
            if (column.isList())
            {
                num++;
                String columnName = column.getColumnName();
                if (columnName.equals(treeName))
                {
                    break;
                }
            }
        }
        return num;
    }
}
